@@ -7,6 +7,7 @@
 
 from typing import Callable, Dict, List, Tuple
 
+from swh.model.identifiers import SWHID, parse_swhid
 import swh.model.model as model
 
 
@@ -16,11 +17,23 @@ def _encode_model_object(obj):
     return d
 
 
+def _encode_model_enum(obj):
+    return {
+        "value": obj.value,
+        "__type__": type(obj).__name__,
+    }
+
+
 ENCODERS: List[Tuple[type, str, Callable]] = [
     (model.BaseModel, "model", _encode_model_object),
+    (SWHID, "swhid", str),
+    (model.MetadataTargetType, "model_enum", _encode_model_enum),
+    (model.MetadataAuthorityType, "model_enum", _encode_model_enum),
 ]
 
 
 DECODERS: Dict[str, Callable] = {
-    "model": lambda d: getattr(model, d.pop("__type__")).from_dict(d)
+    "swhid": parse_swhid,
+    "model": lambda d: getattr(model, d.pop("__type__")).from_dict(d),
+    "model_enum": lambda d: getattr(model, d.pop("__type__"))(d["value"]),
 }
