@@ -8,12 +8,13 @@ import itertools
 import json
 import random
 import re
-from typing import Any, Dict, List, Iterable, Optional, Union
+from typing import Any, Dict, List, Iterable, Optional, Tuple, Union
 
 import attr
-from deprecated import deprecated
 
 from swh.core.api.serializers import msgpack_loads, msgpack_dumps
+from swh.model.identifiers import parse_swhid, SWHID
+from swh.model.hashutil import DEFAULT_ALGORITHMS
 from swh.model.model import (
     Revision,
     Release,
@@ -25,22 +26,19 @@ from swh.model.model import (
     OriginVisitStatus,
     Snapshot,
     Origin,
+    MetadataAuthority,
+    MetadataAuthorityType,
+    MetadataFetcher,
+    MetadataTargetType,
+    RawExtrinsicMetadata,
 )
-from swh.model.hashutil import DEFAULT_ALGORITHMS
 from swh.storage.objstorage import ObjStorage
 from swh.storage.writer import JournalWriter
-from swh.storage.utils import now
+from swh.storage.utils import map_optional, now
 
 from ..exc import StorageArgumentException, HashCollision
-from ..extrinsic_metadata import check_extrinsic_metadata_context, CONTEXT_KEYS
 from .common import TOKEN_BEGIN, TOKEN_END
-from .converters import (
-    revision_to_db,
-    revision_from_db,
-    release_to_db,
-    release_from_db,
-    row_to_visit_status,
-)
+from . import converters
 from .cql import CqlRunner
 from .schema import HASH_ALGORITHMS
 
@@ -430,7 +428,7 @@ class CassandraStorage:
         self.journal_writer.revision_add(revisions)
 
         for revision in revisions:
-            revobject = revision_to_db(revision)
+            revobject = converters.revision_to_db(revision)
             if revobject:
                 # Add parents first
                 for (rank, parent) in enumerate(revobject["parents"]):
@@ -461,7 +459,7 @@ class CassandraStorage:
             # parent_rank is the clustering key, so results are already
             # sorted by rank.
             parents = tuple(row.parent_id for row in parent_rows)
-            rev = revision_from_db(row, parents=parents)
+            rev = converters.revision_from_db(row, parents=parents)
             revs[rev.id] = rev.to_dict()
 
         for rev_id in revisions:
@@ -497,7 +495,7 @@ class CassandraStorage:
             if short:
                 yield (row.id, parents)
             else:
-                rev = revision_from_db(row, parents=parents)
+                rev = converters.revision_from_db(row, parents=parents)
                 yield rev.to_dict()
             yield from self._get_parent_revs(parents, seen, limit, short)
 
@@ -524,7 +522,7 @@ class CassandraStorage:
 
         for release in to_add:
             if release:
-                self._cql_runner.release_add_one(release_to_db(release))
+                self._cql_runner.release_add_one(converters.release_to_db(release))
 
         return {"release:add": len(to_add)}
 
@@ -535,7 +533,7 @@ class CassandraStorage:
         rows = self._cql_runner.release_get(releases)
         rels = {}
         for row in rows:
-            release = release_from_db(row)
+            release = converters.release_from_db(row)
             rels[row.id] = release.to_dict()
 
         for rel_id in releases:
@@ -545,6 +543,7 @@ class CassandraStorage:
         return self._cql_runner.release_get_random().id
 
     def snapshot_add(self, snapshots: Iterable[Snapshot]) -> Dict:
+        snapshots = list(snapshots)
         missing = self._cql_runner.snapshot_missing([snp.id for snp in snapshots])
         snapshots = [snp for snp in snapshots if snp.id in missing]
 
@@ -582,12 +581,12 @@ class CassandraStorage:
         return self.snapshot_get_branches(snapshot_id)
 
     def snapshot_get_by_origin_visit(self, origin, visit):
-        try:
-            visit = self.origin_visit_get_by(origin, visit)
-        except IndexError:
+        visit_status = self.origin_visit_status_get_latest(
+            origin, visit, require_snapshot=True
+        )
+        if not visit_status:
             return None
-
-        return self.snapshot_get(visit["snapshot"])
+        return self.snapshot_get(visit_status.snapshot)
 
     def snapshot_count_branches(self, snapshot_id):
         if self._cql_runner.snapshot_missing([snapshot_id]):
@@ -686,39 +685,17 @@ class CassandraStorage:
 
         return results
 
-    def origin_get(self, origins):
-        if isinstance(origins, dict):
-            # Old API
-            return_single = True
-            origins = [origins]
-        else:
-            return_single = False
+    def origin_get(self, origins: Iterable[str]) -> Iterable[Optional[Origin]]:
+        return [self.origin_get_one(origin) for origin in origins]
 
-        if any("id" in origin for origin in origins):
-            raise StorageArgumentException("Origin ids are not supported.")
+    def origin_get_one(self, origin_url: str) -> Optional[Origin]:
+        """Given an origin url, return the origin if it exists, None otherwise
 
-        results = [self.origin_get_one(origin) for origin in origins]
-
-        if return_single:
-            assert len(results) == 1
-            return results[0]
-        else:
-            return results
-
-    def origin_get_one(self, origin: Dict[str, Any]) -> Optional[Dict[str, Any]]:
-        if "id" in origin:
-            raise StorageArgumentException("Origin ids are not supported.")
-        if "url" not in origin:
-            raise StorageArgumentException("Missing origin url")
-        rows = self._cql_runner.origin_get_by_url(origin["url"])
-
-        rows = list(rows)
+        """
+        rows = list(self._cql_runner.origin_get_by_url(origin_url))
         if rows:
             assert len(rows) == 1
-            result = rows[0]._asdict()
-            return {
-                "url": result["url"],
-            }
+            return Origin(url=rows[0].url)
         else:
             return None
 
@@ -770,34 +747,16 @@ class CassandraStorage:
         return [{"url": orig.url,} for orig in origins[offset : offset + limit]]
 
     def origin_add(self, origins: Iterable[Origin]) -> Dict[str, int]:
-        known_origins = [
-            Origin.from_dict(d)
-            for d in self.origin_get([origin.to_dict() for origin in origins])
-            if d is not None
-        ]
-        to_add = [origin for origin in origins if origin not in known_origins]
+        origins = list(origins)
+        to_add = [ori for ori in origins if self.origin_get_one(ori.url) is None]
         self.journal_writer.origin_add(to_add)
         for origin in to_add:
             self._cql_runner.origin_add_one(origin)
         return {"origin:add": len(to_add)}
 
-    @deprecated("Use origin_add([origin]) instead")
-    def origin_add_one(self, origin: Origin) -> str:
-        known_origin = self.origin_get_one(origin.to_dict())
-
-        if known_origin:
-            origin_url = known_origin["url"]
-        else:
-            self.journal_writer.origin_add([origin])
-
-            self._cql_runner.origin_add_one(origin)
-            origin_url = origin.url
-
-        return origin_url
-
     def origin_visit_add(self, visits: Iterable[OriginVisit]) -> Iterable[OriginVisit]:
         for visit in visits:
-            origin = self.origin_get({"url": visit.origin})
+            origin = self.origin_get_one(visit.origin)
             if not origin:  # Cannot add a visit without an origin
                 raise StorageArgumentException("Unknown origin %s", visit.origin)
 
@@ -836,7 +795,7 @@ class CassandraStorage:
     ) -> None:
         # First round to check existence (fail early if any is ko)
         for visit_status in visit_statuses:
-            origin_url = self.origin_get({"url": visit_status.origin})
+            origin_url = self.origin_get_one(visit_status.origin)
             if not origin_url:
                 raise StorageArgumentException(f"Unknown origin {visit_status.origin}")
 
@@ -852,7 +811,7 @@ class CassandraStorage:
             visit["origin"], visit["visit"]
         )
         assert row is not None
-        visit_status = row_to_visit_status(row)
+        visit_status = converters.row_to_visit_status(row)
         return {
             # default to the values in visit
             **visit,
@@ -865,15 +824,14 @@ class CassandraStorage:
             "date": visit["date"],
         }
 
-    def _origin_visit_get_updated(self, origin: str, visit_id: int) -> Dict[str, Any]:
-        """Retrieve origin visit and latest origin visit status and merge them
-        into an origin visit.
+    def _origin_visit_get_latest_status(self, visit: OriginVisit) -> OriginVisitStatus:
+        """Retrieve the latest visit status information for the origin visit object.
 
         """
-        row_visit = self._cql_runner.origin_visit_get_one(origin, visit_id)
-        assert row_visit is not None
-        visit = self._format_origin_visit_row(row_visit)
-        return self._origin_visit_apply_last_status(visit)
+        row = self._cql_runner.origin_visit_status_get_latest(visit.origin, visit.visit)
+        assert row is not None
+        visit_status = converters.row_to_visit_status(row)
+        return attr.evolve(visit_status, origin=visit.origin)
 
     @staticmethod
     def _format_origin_visit_row(visit):
@@ -898,7 +856,7 @@ class CassandraStorage:
 
     def origin_visit_find_by_date(
         self, origin: str, visit_date: datetime.datetime
-    ) -> Optional[Dict[str, Any]]:
+    ) -> Optional[OriginVisit]:
         # Iterator over all the visits of the origin
         # This should be ok for now, as there aren't too many visits
         # per origin.
@@ -909,16 +867,13 @@ class CassandraStorage:
             return (abs(dt), -visit.visit)
 
         if rows:
-            row = min(rows, key=key)
-            visit = self._format_origin_visit_row(row)
-            return self._origin_visit_apply_last_status(visit)
+            return converters.row_to_visit(min(rows, key=key))
         return None
 
-    def origin_visit_get_by(self, origin: str, visit: int) -> Optional[Dict[str, Any]]:
+    def origin_visit_get_by(self, origin: str, visit: int) -> Optional[OriginVisit]:
         row = self._cql_runner.origin_visit_get_one(origin, visit)
         if row:
-            visit_ = self._format_origin_visit_row(row)
-            return self._origin_visit_apply_last_status(visit_)
+            return converters.row_to_visit(row)
         return None
 
     def origin_visit_get_latest(
@@ -927,7 +882,7 @@ class CassandraStorage:
         type: Optional[str] = None,
         allowed_statuses: Optional[List[str]] = None,
         require_snapshot: bool = False,
-    ) -> Optional[Dict[str, Any]]:
+    ) -> Optional[OriginVisit]:
         # TODO: Do not fetch all visits
         rows = self._cql_runner.origin_visit_get_all(origin)
         latest_visit = None
@@ -950,7 +905,14 @@ class CassandraStorage:
 
             latest_visit = updated_visit
 
-        return latest_visit
+        if latest_visit is None:
+            return None
+        return OriginVisit(
+            origin=latest_visit["origin"],
+            visit=latest_visit["visit"],
+            date=latest_visit["date"],
+            type=latest_visit["type"],
+        )
 
     def origin_visit_status_get_latest(
         self,
@@ -969,9 +931,11 @@ class CassandraStorage:
             rows = [row for row in rows if row.snapshot is not None]
         if not rows:
             return None
-        return row_to_visit_status(rows[0])
+        return converters.row_to_visit_status(rows[0])
 
-    def origin_visit_get_random(self, type: str) -> Optional[Dict[str, Any]]:
+    def origin_visit_status_get_random(
+        self, type: str
+    ) -> Optional[Tuple[OriginVisit, OriginVisitStatus]]:
         back_in_the_day = now() - datetime.timedelta(weeks=12)  # 3 months back
 
         # Random position to start iteration at
@@ -980,15 +944,11 @@ class CassandraStorage:
         # Iterator over all visits, ordered by token(origins) then visit_id
         rows = self._cql_runner.origin_visit_iter(start_token)
         for row in rows:
-            visit = self._format_origin_visit_row(row)
-            visit_status = self._origin_visit_apply_last_status(visit)
-            if (
-                visit_status["date"] > back_in_the_day
-                and visit_status["status"] == "full"
-            ):
-                return visit_status
-        else:
-            return None
+            visit = converters.row_to_visit(row)
+            visit_status = self._origin_visit_get_latest_status(visit)
+            if visit.date > back_in_the_day and visit_status.status == "full":
+                return visit, visit_status
+        return None
 
     def stat_counters(self):
         rows = self._cql_runner.stat_counters()
@@ -1009,128 +969,67 @@ class CassandraStorage:
     def refresh_stat_counters(self):
         pass
 
-    def content_metadata_add(
-        self,
-        id: str,
-        context: Dict[str, Union[str, bytes, int]],
-        discovery_date: datetime.datetime,
-        authority: Dict[str, Any],
-        fetcher: Dict[str, Any],
-        format: str,
-        metadata: bytes,
+    def raw_extrinsic_metadata_add(
+        self, metadata: Iterable[RawExtrinsicMetadata]
     ) -> None:
-        self._object_metadata_add(
-            "content",
-            id,
-            discovery_date,
-            authority,
-            fetcher,
-            format,
-            metadata,
-            context,
-        )
+        for metadata_entry in metadata:
+            if not self._cql_runner.metadata_authority_get(
+                metadata_entry.authority.type.value, metadata_entry.authority.url
+            ):
+                raise StorageArgumentException(
+                    f"Unknown authority {metadata_entry.authority}"
+                )
+            if not self._cql_runner.metadata_fetcher_get(
+                metadata_entry.fetcher.name, metadata_entry.fetcher.version
+            ):
+                raise StorageArgumentException(
+                    f"Unknown fetcher {metadata_entry.fetcher}"
+                )
 
-    def content_metadata_get(
+            try:
+                self._cql_runner.raw_extrinsic_metadata_add(
+                    type=metadata_entry.type.value,
+                    id=str(metadata_entry.id),
+                    authority_type=metadata_entry.authority.type.value,
+                    authority_url=metadata_entry.authority.url,
+                    discovery_date=metadata_entry.discovery_date,
+                    fetcher_name=metadata_entry.fetcher.name,
+                    fetcher_version=metadata_entry.fetcher.version,
+                    format=metadata_entry.format,
+                    metadata=metadata_entry.metadata,
+                    origin=metadata_entry.origin,
+                    visit=metadata_entry.visit,
+                    snapshot=map_optional(str, metadata_entry.snapshot),
+                    release=map_optional(str, metadata_entry.release),
+                    revision=map_optional(str, metadata_entry.revision),
+                    path=metadata_entry.path,
+                    directory=map_optional(str, metadata_entry.directory),
+                )
+            except TypeError as e:
+                raise StorageArgumentException(*e.args)
+
+    def raw_extrinsic_metadata_get(
         self,
-        id: str,
-        authority: Dict[str, str],
+        object_type: MetadataTargetType,
+        id: Union[str, SWHID],
+        authority: MetadataAuthority,
         after: Optional[datetime.datetime] = None,
         page_token: Optional[bytes] = None,
         limit: int = 1000,
-    ) -> Dict[str, Any]:
-        return self._object_metadata_get(
-            "content", id, authority, after, page_token, limit,
-        )
+    ) -> Dict[str, Union[Optional[bytes], List[RawExtrinsicMetadata]]]:
+        if object_type == MetadataTargetType.ORIGIN:
+            if isinstance(id, SWHID):
+                raise StorageArgumentException(
+                    f"raw_extrinsic_metadata_get called with object_type='origin', "
+                    f"but provided id is an SWHID: {id!r}"
+                )
+        else:
+            if not isinstance(id, SWHID):
+                raise StorageArgumentException(
+                    f"raw_extrinsic_metadata_get called with object_type!='origin', "
+                    f"but provided id is not an SWHID: {id!r}"
+                )
 
-    def origin_metadata_add(
-        self,
-        origin_url: str,
-        discovery_date: datetime.datetime,
-        authority: Dict[str, Any],
-        fetcher: Dict[str, Any],
-        format: str,
-        metadata: bytes,
-    ) -> None:
-        if not isinstance(origin_url, str):
-            raise StorageArgumentException(
-                "origin_url must be str, not %r" % (origin_url,)
-            )
-
-        context: Dict[str, Union[str, bytes, int]] = {}  # origins have no context
-
-        self._object_metadata_add(
-            "origin",
-            origin_url,
-            discovery_date,
-            authority,
-            fetcher,
-            format,
-            metadata,
-            context,
-        )
-
-    def origin_metadata_get(
-        self,
-        origin_url: str,
-        authority: Dict[str, str],
-        after: Optional[datetime.datetime] = None,
-        page_token: Optional[bytes] = None,
-        limit: int = 1000,
-    ) -> Dict[str, Any]:
-        if not isinstance(origin_url, str):
-            raise TypeError("origin_url must be str, not %r" % (origin_url,))
-
-        res = self._object_metadata_get(
-            "origin", origin_url, authority, after, page_token, limit
-        )
-        for result in res["results"]:
-            result["origin_url"] = result.pop("id")
-
-        return res
-
-    def _object_metadata_add(
-        self,
-        object_type: str,
-        id: str,
-        discovery_date: datetime.datetime,
-        authority: Dict[str, Any],
-        fetcher: Dict[str, Any],
-        format: str,
-        metadata: bytes,
-        context: Dict[str, Union[str, bytes, int]],
-    ) -> None:
-        check_extrinsic_metadata_context(object_type, context)
-
-        if not self._cql_runner.metadata_authority_get(**authority):
-            raise StorageArgumentException(f"Unknown authority {authority}")
-        if not self._cql_runner.metadata_fetcher_get(**fetcher):
-            raise StorageArgumentException(f"Unknown fetcher {fetcher}")
-
-        try:
-            self._cql_runner.object_metadata_add(
-                object_type,
-                id,
-                authority["type"],
-                authority["url"],
-                discovery_date,
-                fetcher["name"],
-                fetcher["version"],
-                format,
-                metadata,
-                context,
-            )
-        except TypeError as e:
-            raise StorageArgumentException(*e.args)
-
-    def _object_metadata_get(
-        self,
-        object_type: str,
-        id: str,
-        authority: Dict[str, str],
-        after: Optional[datetime.datetime] = None,
-        page_token: Optional[bytes] = None,
-        limit: int = 1000,
-    ) -> Dict[str, Any]:
         if page_token is not None:
             (after_date, after_fetcher_name, after_fetcher_url) = msgpack_loads(
                 page_token
@@ -1139,21 +1038,21 @@ class CassandraStorage:
                 raise StorageArgumentException(
                     "page_token is inconsistent with the value of 'after'."
                 )
-            entries = self._cql_runner.object_metadata_get_after_date_and_fetcher(
-                id,
-                authority["type"],
-                authority["url"],
+            entries = self._cql_runner.raw_extrinsic_metadata_get_after_date_and_fetcher(  # noqa
+                str(id),
+                authority.type.value,
+                authority.url,
                 after_date,
                 after_fetcher_name,
                 after_fetcher_url,
             )
         elif after is not None:
-            entries = self._cql_runner.object_metadata_get_after_date(
-                id, authority["type"], authority["url"], after
+            entries = self._cql_runner.raw_extrinsic_metadata_get_after_date(
+                str(id), authority.type.value, authority.url, after
             )
         else:
-            entries = self._cql_runner.object_metadata_get(
-                id, authority["type"], authority["url"]
+            entries = self._cql_runner.raw_extrinsic_metadata_get(
+                str(id), authority.type.value, authority.url
             )
 
         if limit:
@@ -1163,28 +1062,29 @@ class CassandraStorage:
         for entry in entries:
             discovery_date = entry.discovery_date.replace(tzinfo=datetime.timezone.utc)
 
-            result = {
-                "id": entry.id,
-                "authority": {
-                    "type": entry.authority_type,
-                    "url": entry.authority_url,
-                },
-                "fetcher": {
-                    "name": entry.fetcher_name,
-                    "version": entry.fetcher_version,
-                },
-                "discovery_date": discovery_date,
-                "format": entry.format,
-                "metadata": entry.metadata,
-            }
+            assert str(id) == entry.id
 
-            if CONTEXT_KEYS[object_type]:
-                context = {}
-                for key in CONTEXT_KEYS[object_type]:
-                    value = getattr(entry, key)
-                    if value is not None:
-                        context[key] = value
-                result["context"] = context
+            result = RawExtrinsicMetadata(
+                type=MetadataTargetType(entry.type),
+                id=id,
+                authority=MetadataAuthority(
+                    type=MetadataAuthorityType(entry.authority_type),
+                    url=entry.authority_url,
+                ),
+                fetcher=MetadataFetcher(
+                    name=entry.fetcher_name, version=entry.fetcher_version,
+                ),
+                discovery_date=discovery_date,
+                format=entry.format,
+                metadata=entry.metadata,
+                origin=entry.origin,
+                visit=entry.visit,
+                snapshot=map_optional(parse_swhid, entry.snapshot),
+                release=map_optional(parse_swhid, entry.release),
+                revision=map_optional(parse_swhid, entry.revision),
+                path=entry.path,
+                directory=map_optional(parse_swhid, entry.directory),
+            )
 
             results.append(result)
 
@@ -1194,9 +1094,9 @@ class CassandraStorage:
             last_result = results[-1]
             next_page_token: Optional[bytes] = msgpack_dumps(
                 (
-                    last_result["discovery_date"],
-                    last_result["fetcher"]["name"],
-                    last_result["fetcher"]["version"],
+                    last_result.discovery_date,
+                    last_result.fetcher.name,
+                    last_result.fetcher.version,
                 )
             )
         else:
@@ -1207,35 +1107,45 @@ class CassandraStorage:
             "results": results,
         }
 
-    def metadata_fetcher_add(
-        self, name: str, version: str, metadata: Dict[str, Any]
-    ) -> None:
-        self._cql_runner.metadata_fetcher_add(name, version, json.dumps(metadata))
+    def metadata_fetcher_add(self, fetchers: Iterable[MetadataFetcher]) -> None:
+        for fetcher in fetchers:
+            self._cql_runner.metadata_fetcher_add(
+                fetcher.name,
+                fetcher.version,
+                json.dumps(map_optional(dict, fetcher.metadata)),
+            )
 
-    def metadata_fetcher_get(self, name: str, version: str) -> Optional[Dict[str, Any]]:
+    def metadata_fetcher_get(
+        self, name: str, version: str
+    ) -> Optional[MetadataFetcher]:
         fetcher = self._cql_runner.metadata_fetcher_get(name, version)
         if fetcher:
-            return {
-                "name": fetcher.name,
-                "version": fetcher.version,
-                "metadata": json.loads(fetcher.metadata),
-            }
+            return MetadataFetcher(
+                name=fetcher.name,
+                version=fetcher.version,
+                metadata=json.loads(fetcher.metadata),
+            )
         else:
             return None
 
-    def metadata_authority_add(
-        self, type: str, url: str, metadata: Dict[str, Any]
-    ) -> None:
-        self._cql_runner.metadata_authority_add(url, type, json.dumps(metadata))
+    def metadata_authority_add(self, authorities: Iterable[MetadataAuthority]) -> None:
+        for authority in authorities:
+            self._cql_runner.metadata_authority_add(
+                authority.url,
+                authority.type.value,
+                json.dumps(map_optional(dict, authority.metadata)),
+            )
 
-    def metadata_authority_get(self, type: str, url: str) -> Optional[Dict[str, Any]]:
-        authority = self._cql_runner.metadata_authority_get(type, url)
+    def metadata_authority_get(
+        self, type: MetadataAuthorityType, url: str
+    ) -> Optional[MetadataAuthority]:
+        authority = self._cql_runner.metadata_authority_get(type.value, url)
         if authority:
-            return {
-                "type": authority.type,
-                "url": authority.url,
-                "metadata": json.loads(authority.metadata),
-            }
+            return MetadataAuthority(
+                type=MetadataAuthorityType(authority.type),
+                url=authority.url,
+                metadata=json.loads(authority.metadata),
+            )
         else:
             return None
 
