@@ -30,9 +30,8 @@ from typing import (
 
 import attr
 
-from deprecated import deprecated
-
 from swh.core.api.serializers import msgpack_loads, msgpack_dumps
+from swh.model.identifiers import SWHID
 from swh.model.model import (
     BaseContent,
     Content,
@@ -45,6 +44,11 @@ from swh.model.model import (
     OriginVisitStatus,
     Origin,
     SHA1_SIZE,
+    MetadataAuthority,
+    MetadataAuthorityType,
+    MetadataFetcher,
+    MetadataTargetType,
+    RawExtrinsicMetadata,
 )
 from swh.model.hashutil import DEFAULT_ALGORITHMS, hash_to_bytes, hash_to_hex
 from swh.storage.objstorage import ObjStorage
@@ -52,7 +56,6 @@ from swh.storage.utils import now
 
 from .converters import origin_url_to_sha1
 from .exc import StorageArgumentException, HashCollision
-from .extrinsic_metadata import check_extrinsic_metadata_context, CONTEXT_KEYS
 from .utils import get_partition_bounds_bytes
 from .writer import JournalWriter
 
@@ -138,21 +141,33 @@ class InMemoryStorage:
         self._origin_visit_statuses: Dict[Tuple[str, int], List[OriginVisitStatus]] = {}
         self._persons = {}
 
-        # {origin_url: {authority: [metadata]}}
-        self._object_metadata: Dict[
-            str,
+        # {object_type: {id: {authority: [metadata]}}}
+        self._raw_extrinsic_metadata: Dict[
+            MetadataTargetType,
             Dict[
-                Hashable,
-                SortedList[Tuple[datetime.datetime, FetcherKey], Dict[str, Any]],
+                Union[str, SWHID],
+                Dict[
+                    Hashable,
+                    SortedList[
+                        Tuple[datetime.datetime, FetcherKey], RawExtrinsicMetadata
+                    ],
+                ],
             ],
         ] = defaultdict(
             lambda: defaultdict(
-                lambda: SortedList(key=lambda x: (x["discovery_date"], x["fetcher"]))
+                lambda: defaultdict(
+                    lambda: SortedList(
+                        key=lambda x: (
+                            x.discovery_date,
+                            self._metadata_fetcher_key(x.fetcher),
+                        )
+                    )
+                )
             )
         )  # noqa
 
-        self._metadata_fetchers: Dict[FetcherKey, Dict[str, Any]] = {}
-        self._metadata_authorities: Dict[Hashable, Dict[str, Any]] = {}
+        self._metadata_fetchers: Dict[FetcherKey, MetadataFetcher] = {}
+        self._metadata_authorities: Dict[Hashable, MetadataAuthority] = {}
         self._objects = defaultdict(list)
         self._sorted_sha1s = SortedList[bytes, bytes]()
 
@@ -655,43 +670,11 @@ class InMemoryStorage:
 
         return t.to_dict()
 
-    def origin_get(self, origins):
-        if isinstance(origins, dict):
-            # Old API
-            return_single = True
-            origins = [origins]
-        else:
-            return_single = False
+    def origin_get_one(self, origin_url: str) -> Optional[Origin]:
+        return self._origins.get(origin_url)
 
-        # Sanity check to be error-compatible with the pgsql backend
-        if any("id" in origin for origin in origins) and not all(
-            "id" in origin for origin in origins
-        ):
-            raise StorageArgumentException(
-                'Either all origins or none at all should have an "id".'
-            )
-        if any("url" in origin for origin in origins) and not all(
-            "url" in origin for origin in origins
-        ):
-            raise StorageArgumentException(
-                "Either all origins or none at all should have " 'an "url" key.'
-            )
-
-        results = []
-        for origin in origins:
-            result = None
-            if "url" in origin:
-                if origin["url"] in self._origins:
-                    result = self._origins[origin["url"]]
-            else:
-                raise StorageArgumentException("Origin must have an url.")
-            results.append(self._convert_origin(result))
-
-        if return_single:
-            assert len(results) == 1
-            return results[0]
-        else:
-            return results
+    def origin_get(self, origins: Iterable[str]) -> Iterable[Optional[Origin]]:
+        return [self.origin_get_one(origin_url) for origin_url in origins]
 
     def origin_get_by_sha1(self, sha1s):
         return [self._convert_origin(self._origins_by_sha1.get(sha1)) for sha1 in sha1s]
@@ -770,7 +753,6 @@ class InMemoryStorage:
 
         return {"origin:add": added}
 
-    @deprecated("Use origin_add([origin]) instead")
     def origin_add_one(self, origin: Origin) -> str:
         if origin.url not in self._origins:
             self.journal_writer.origin_add([origin])
@@ -789,7 +771,7 @@ class InMemoryStorage:
 
     def origin_visit_add(self, visits: Iterable[OriginVisit]) -> Iterable[OriginVisit]:
         for visit in visits:
-            origin = self.origin_get({"url": visit.origin})
+            origin = self.origin_get_one(visit.origin)
             if not origin:  # Cannot add a visit without an origin
                 raise StorageArgumentException("Unknown origin %s", visit.origin)
 
@@ -841,15 +823,17 @@ class InMemoryStorage:
     ) -> None:
         # First round to check existence (fail early if any is ko)
         for visit_status in visit_statuses:
-            origin_url = self.origin_get({"url": visit_status.origin})
+            origin_url = self.origin_get_one(visit_status.origin)
             if not origin_url:
                 raise StorageArgumentException(f"Unknown origin {visit_status.origin}")
 
         for visit_status in visit_statuses:
             self._origin_visit_status_add_one(visit_status)
 
-    def _origin_visit_get_updated(self, origin: str, visit_id: int) -> Dict[str, Any]:
-        """Merge origin visit and latest origin visit status
+    def _origin_visit_status_get_latest(
+        self, origin: str, visit_id: int
+    ) -> Tuple[OriginVisit, OriginVisitStatus]:
+        """Return a tuple of OriginVisit, latest associated OriginVisitStatus.
 
         """
         assert visit_id >= 1
@@ -858,6 +842,14 @@ class InMemoryStorage:
         visit_key = (origin, visit_id)
 
         visit_update = max(self._origin_visit_statuses[visit_key], key=lambda v: v.date)
+        return visit, visit_update
+
+    def _origin_visit_get_updated(self, origin: str, visit_id: int) -> Dict[str, Any]:
+        """Merge origin visit and latest origin visit status
+
+        """
+        visit, visit_update = self._origin_visit_status_get_latest(origin, visit_id)
+        assert visit is not None and visit_update is not None
         return {
             # default to the values in visit
             **visit.to_dict(),
@@ -898,24 +890,21 @@ class InMemoryStorage:
 
     def origin_visit_find_by_date(
         self, origin: str, visit_date: datetime.datetime
-    ) -> Optional[Dict[str, Any]]:
+    ) -> Optional[OriginVisit]:
         origin_url = self._get_origin_url(origin)
         if origin_url in self._origin_visits:
             visits = self._origin_visits[origin_url]
             visit = min(visits, key=lambda v: (abs(v.date - visit_date), -v.visit))
-            visit_update = self._origin_visit_get_updated(origin, visit.visit)
-            assert visit_update is not None
-            return visit_update
+            return visit
         return None
 
-    def origin_visit_get_by(self, origin: str, visit: int) -> Optional[Dict[str, Any]]:
+    def origin_visit_get_by(self, origin: str, visit: int) -> Optional[OriginVisit]:
         origin_url = self._get_origin_url(origin)
         if origin_url in self._origin_visits and visit <= len(
             self._origin_visits[origin_url]
         ):
-            visit_update = self._origin_visit_get_updated(origin_url, visit)
-            assert visit_update is not None
-            return visit_update
+            found_visit, _ = self._origin_visit_status_get_latest(origin, visit)
+            return found_visit
         return None
 
     def origin_visit_get_latest(
@@ -924,29 +913,33 @@ class InMemoryStorage:
         type: Optional[str] = None,
         allowed_statuses: Optional[List[str]] = None,
         require_snapshot: bool = False,
-    ) -> Optional[Dict[str, Any]]:
+    ) -> Optional[OriginVisit]:
         ori = self._origins.get(origin)
         if not ori:
             return None
-        visits = self._origin_visits[ori.url]
 
-        visits = [
-            self._origin_visit_get_updated(visit.origin, visit.visit)
-            for visit in visits
-            if visit is not None
-        ]
+        visits = sorted(
+            self._origin_visits[ori.url], key=lambda v: v.date, reverse=True,
+        )
+        for visit in visits:
+            if type is not None and visit.type != type:
+                continue
+            visit_statuses = self._origin_visit_statuses[origin, visit.visit]
 
-        if type is not None:
-            visits = [visit for visit in visits if visit["type"] == type]
-        if allowed_statuses is not None:
-            visits = [visit for visit in visits if visit["status"] in allowed_statuses]
-        if require_snapshot:
-            visits = [visit for visit in visits if visit["snapshot"]]
+            if allowed_statuses is not None:
+                visit_statuses = [
+                    vs for vs in visit_statuses if vs.status in allowed_statuses
+                ]
+            if require_snapshot:
+                visit_statuses = [vs for vs in visit_statuses if vs.snapshot]
 
-        visit = max(visits, key=lambda v: (v["date"], v["visit"]), default=None)
-        if visit is None:
-            return None
-        return visit
+            if visit_statuses:  # we found visit statuses matching criteria
+                visit_status = max(visit_statuses, key=lambda vs: (vs.date, vs.visit))
+                assert visit.origin == visit_status.origin
+                assert visit.visit == visit_status.visit
+                return visit
+
+        return None
 
     def origin_visit_status_get_latest(
         self,
@@ -979,20 +972,25 @@ class InMemoryStorage:
             if random_origin_visits[0].type == type:
                 return url
 
-    def origin_visit_get_random(self, type: str) -> Optional[Dict[str, Any]]:
+    def origin_visit_status_get_random(
+        self, type: str
+    ) -> Optional[Tuple[OriginVisit, OriginVisitStatus]]:
+
         url = self._select_random_origin_visit_by_type(type)
         random_origin_visits = copy.deepcopy(self._origin_visits[url])
         random_origin_visits.reverse()
         back_in_the_day = now() - timedelta(weeks=12)  # 3 months back
         # This should be enough for tests
         for visit in random_origin_visits:
-            updated_visit = self._origin_visit_get_updated(url, visit.visit)
-            assert updated_visit is not None
+            origin_visit, latest_visit_status = self._origin_visit_status_get_latest(
+                url, visit.visit
+            )
+            assert latest_visit_status is not None
             if (
-                updated_visit["date"] > back_in_the_day
-                and updated_visit["status"] == "full"
+                origin_visit.date > back_in_the_day
+                and latest_visit_status.status == "full"
             ):
-                return updated_visit
+                return origin_visit, latest_visit_status
         else:
             return None
 
@@ -1020,144 +1018,60 @@ class InMemoryStorage:
     def refresh_stat_counters(self):
         pass
 
-    def content_metadata_add(
-        self,
-        id: str,
-        context: Dict[str, Union[str, bytes, int]],
-        discovery_date: datetime.datetime,
-        authority: Dict[str, Any],
-        fetcher: Dict[str, Any],
-        format: str,
-        metadata: bytes,
+    def raw_extrinsic_metadata_add(
+        self, metadata: Iterable[RawExtrinsicMetadata],
     ) -> None:
-        self._object_metadata_add(
-            "content",
-            id,
-            discovery_date,
-            authority,
-            fetcher,
-            format,
-            metadata,
-            context,
-        )
+        for metadata_entry in metadata:
+            authority_key = self._metadata_authority_key(metadata_entry.authority)
+            if authority_key not in self._metadata_authorities:
+                raise StorageArgumentException(
+                    f"Unknown authority {metadata_entry.authority}"
+                )
+            fetcher_key = self._metadata_fetcher_key(metadata_entry.fetcher)
+            if fetcher_key not in self._metadata_fetchers:
+                raise StorageArgumentException(
+                    f"Unknown fetcher {metadata_entry.fetcher}"
+                )
 
-    def content_metadata_get(
+            raw_extrinsic_metadata_list = self._raw_extrinsic_metadata[
+                metadata_entry.type
+            ][metadata_entry.id][authority_key]
+
+            for existing_raw_extrinsic_metadata in raw_extrinsic_metadata_list:
+                if (
+                    self._metadata_fetcher_key(existing_raw_extrinsic_metadata.fetcher)
+                    == fetcher_key
+                    and existing_raw_extrinsic_metadata.discovery_date
+                    == metadata_entry.discovery_date
+                ):
+                    # Duplicate of an existing one; ignore it.
+                    break
+            else:
+                raw_extrinsic_metadata_list.add(metadata_entry)
+
+    def raw_extrinsic_metadata_get(
         self,
-        id: str,
-        authority: Dict[str, str],
+        object_type: MetadataTargetType,
+        id: Union[str, SWHID],
+        authority: MetadataAuthority,
         after: Optional[datetime.datetime] = None,
         page_token: Optional[bytes] = None,
         limit: int = 1000,
-    ) -> Dict[str, Any]:
-        return self._object_metadata_get(
-            "content", id, authority, after, page_token, limit
-        )
-
-    def origin_metadata_add(
-        self,
-        origin_url: str,
-        discovery_date: datetime.datetime,
-        authority: Dict[str, Any],
-        fetcher: Dict[str, Any],
-        format: str,
-        metadata: bytes,
-    ) -> None:
-        if not isinstance(origin_url, str):
-            raise StorageArgumentException(
-                "origin_url must be str, not %r" % (origin_url,)
-            )
-
-        context: Dict[str, Union[str, bytes, int]] = {}  # origins have no context
-
-        self._object_metadata_add(
-            "origin",
-            origin_url,
-            discovery_date,
-            authority,
-            fetcher,
-            format,
-            metadata,
-            context,
-        )
-
-    def origin_metadata_get(
-        self,
-        origin_url: str,
-        authority: Dict[str, str],
-        after: Optional[datetime.datetime] = None,
-        page_token: Optional[bytes] = None,
-        limit: int = 1000,
-    ) -> Dict[str, Any]:
-        if not isinstance(origin_url, str):
-            raise TypeError("origin_url must be str, not %r" % (origin_url,))
-
-        res = self._object_metadata_get(
-            "origin", origin_url, authority, after, page_token, limit
-        )
-        res["results"] = copy.deepcopy(res["results"])
-        for result in res["results"]:
-            result["origin_url"] = result.pop("id")
-
-        return res
-
-    def _object_metadata_add(
-        self,
-        object_type: str,
-        id: str,
-        discovery_date: datetime.datetime,
-        authority: Dict[str, Any],
-        fetcher: Dict[str, Any],
-        format: str,
-        metadata: bytes,
-        context: Dict[str, Union[str, bytes, int]],
-    ) -> None:
-        check_extrinsic_metadata_context(object_type, context)
-        if not isinstance(metadata, bytes):
-            raise StorageArgumentException(
-                "metadata must be bytes, not %r" % (metadata,)
-            )
+    ) -> Dict[str, Union[Optional[bytes], List[RawExtrinsicMetadata]]]:
         authority_key = self._metadata_authority_key(authority)
-        if authority_key not in self._metadata_authorities:
-            raise StorageArgumentException(f"Unknown authority {authority}")
-        fetcher_key = self._metadata_fetcher_key(fetcher)
-        if fetcher_key not in self._metadata_fetchers:
-            raise StorageArgumentException(f"Unknown fetcher {fetcher}")
 
-        object_metadata_list = self._object_metadata[id][authority_key]
-
-        object_metadata: Dict[str, Any] = {
-            "id": id,
-            "discovery_date": discovery_date,
-            "authority": authority_key,
-            "fetcher": fetcher_key,
-            "format": format,
-            "metadata": metadata,
-        }
-
-        if CONTEXT_KEYS[object_type]:
-            object_metadata["context"] = context
-
-        for existing_object_metadata in object_metadata_list:
-            if (
-                existing_object_metadata["fetcher"] == fetcher_key
-                and existing_object_metadata["discovery_date"] == discovery_date
-            ):
-                # Duplicate of an existing one; replace it.
-                existing_object_metadata.update(object_metadata)
-                break
+        if object_type == MetadataTargetType.ORIGIN:
+            if isinstance(id, SWHID):
+                raise StorageArgumentException(
+                    f"raw_extrinsic_metadata_get called with object_type='origin', "
+                    f"but provided id is an SWHID: {id!r}"
+                )
         else:
-            object_metadata_list.add(object_metadata)
-
-    def _object_metadata_get(
-        self,
-        object_type: str,
-        id: str,
-        authority: Dict[str, str],
-        after: Optional[datetime.datetime] = None,
-        page_token: Optional[bytes] = None,
-        limit: int = 1000,
-    ) -> Dict[str, Any]:
-        authority_key = self._metadata_authority_key(authority)
+            if not isinstance(id, SWHID):
+                raise StorageArgumentException(
+                    f"raw_extrinsic_metadata_get called with object_type!='origin', "
+                    f"but provided id is not an SWHID: {id!r}"
+                )
 
         if page_token is not None:
             (after_time, after_fetcher) = msgpack_loads(page_token)
@@ -1166,33 +1080,36 @@ class InMemoryStorage:
                 raise StorageArgumentException(
                     "page_token is inconsistent with the value of 'after'."
                 )
-            entries = self._object_metadata[id][authority_key].iter_after(
-                (after_time, after_fetcher)
-            )
+            entries = self._raw_extrinsic_metadata[object_type][id][
+                authority_key
+            ].iter_after((after_time, after_fetcher))
         elif after is not None:
-            entries = self._object_metadata[id][authority_key].iter_from((after,))
-            entries = (entry for entry in entries if entry["discovery_date"] > after)
+            entries = self._raw_extrinsic_metadata[object_type][id][
+                authority_key
+            ].iter_from((after,))
+            entries = (entry for entry in entries if entry.discovery_date > after)
         else:
-            entries = iter(self._object_metadata[id][authority_key])
+            entries = iter(self._raw_extrinsic_metadata[object_type][id][authority_key])
 
         if limit:
             entries = itertools.islice(entries, 0, limit + 1)
 
         results = []
         for entry in entries:
-            authority = self._metadata_authorities[entry["authority"]]
-            fetcher = self._metadata_fetchers[entry["fetcher"]]
+            entry_authority = self._metadata_authorities[
+                self._metadata_authority_key(entry.authority)
+            ]
+            entry_fetcher = self._metadata_fetchers[
+                self._metadata_fetcher_key(entry.fetcher)
+            ]
             if after:
-                assert entry["discovery_date"] > after
+                assert entry.discovery_date > after
             results.append(
-                {
-                    **entry,
-                    "authority": {"type": authority["type"], "url": authority["url"],},
-                    "fetcher": {
-                        "name": fetcher["name"],
-                        "version": fetcher["version"],
-                    },
-                }
+                attr.evolve(
+                    entry,
+                    authority=attr.evolve(entry_authority, metadata=None),
+                    fetcher=attr.evolve(entry_fetcher, metadata=None),
+                )
             )
 
         if len(results) > limit:
@@ -1201,8 +1118,8 @@ class InMemoryStorage:
             last_result = results[-1]
             next_page_token: Optional[bytes] = msgpack_dumps(
                 (
-                    last_result["discovery_date"],
-                    self._metadata_fetcher_key(last_result["fetcher"]),
+                    last_result.discovery_date,
+                    self._metadata_fetcher_key(last_result.fetcher),
                 )
             )
         else:
@@ -1213,37 +1130,38 @@ class InMemoryStorage:
             "results": results,
         }
 
-    def metadata_fetcher_add(
-        self, name: str, version: str, metadata: Dict[str, Any]
-    ) -> None:
-        fetcher = {
-            "name": name,
-            "version": version,
-            "metadata": metadata,
-        }
-        key = self._metadata_fetcher_key(fetcher)
-        if key not in self._metadata_fetchers:
-            self._metadata_fetchers[key] = fetcher
+    def metadata_fetcher_add(self, fetchers: Iterable[MetadataFetcher]) -> None:
+        for fetcher in fetchers:
+            if fetcher.metadata is None:
+                raise StorageArgumentException(
+                    "MetadataFetcher.metadata may not be None in metadata_fetcher_add."
+                )
+            key = self._metadata_fetcher_key(fetcher)
+            if key not in self._metadata_fetchers:
+                self._metadata_fetchers[key] = fetcher
 
-    def metadata_fetcher_get(self, name: str, version: str) -> Optional[Dict[str, Any]]:
+    def metadata_fetcher_get(
+        self, name: str, version: str
+    ) -> Optional[MetadataFetcher]:
         return self._metadata_fetchers.get(
-            self._metadata_fetcher_key({"name": name, "version": version})
+            self._metadata_fetcher_key(MetadataFetcher(name=name, version=version))
         )
 
-    def metadata_authority_add(
-        self, type: str, url: str, metadata: Dict[str, Any]
-    ) -> None:
-        authority = {
-            "type": type,
-            "url": url,
-            "metadata": metadata,
-        }
-        key = self._metadata_authority_key(authority)
-        self._metadata_authorities[key] = authority
+    def metadata_authority_add(self, authorities: Iterable[MetadataAuthority]) -> None:
+        for authority in authorities:
+            if authority.metadata is None:
+                raise StorageArgumentException(
+                    "MetadataAuthority.metadata may not be None in "
+                    "metadata_authority_add."
+                )
+            key = self._metadata_authority_key(authority)
+            self._metadata_authorities[key] = authority
 
-    def metadata_authority_get(self, type: str, url: str) -> Optional[Dict[str, Any]]:
+    def metadata_authority_get(
+        self, type: MetadataAuthorityType, url: str
+    ) -> Optional[MetadataAuthority]:
         return self._metadata_authorities.get(
-            self._metadata_authority_key({"type": type, "url": url})
+            self._metadata_authority_key(MetadataAuthority(type=type, url=url))
         )
 
     def _get_origin_url(self, origin):
@@ -1268,12 +1186,12 @@ class InMemoryStorage:
         return tuple((key, content.get(key)) for key in sorted(DEFAULT_ALGORITHMS))
 
     @staticmethod
-    def _metadata_fetcher_key(fetcher: Dict) -> FetcherKey:
-        return (fetcher["name"], fetcher["version"])
+    def _metadata_fetcher_key(fetcher: MetadataFetcher) -> FetcherKey:
+        return (fetcher.name, fetcher.version)
 
     @staticmethod
-    def _metadata_authority_key(authority: Dict) -> Hashable:
-        return (authority["type"], authority["url"])
+    def _metadata_authority_key(authority: MetadataAuthority) -> Hashable:
+        return (authority.type, authority.url)
 
     def diff_directories(self, from_dir, to_dir, track_renaming=False):
         raise NotImplementedError("InMemoryStorage.diff_directories")
