@@ -5,9 +5,10 @@
 
 import datetime
 
-from typing import Any, Dict, Iterable, List, Optional, Union
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 
 from swh.core.api import remote_api_endpoint
+from swh.model.identifiers import SWHID
 from swh.model.model import (
     Content,
     Directory,
@@ -18,6 +19,11 @@ from swh.model.model import (
     Release,
     Snapshot,
     SkippedContent,
+    MetadataAuthority,
+    MetadataAuthorityType,
+    MetadataFetcher,
+    MetadataTargetType,
+    RawExtrinsicMetadata,
 )
 
 
@@ -810,7 +816,7 @@ class StorageInterface:
     @remote_api_endpoint("origin/visit/find_by_date")
     def origin_visit_find_by_date(
         self, origin: str, visit_date: datetime.datetime
-    ) -> Optional[Dict[str, Any]]:
+    ) -> Optional[OriginVisit]:
         """Retrieves the origin visit whose date is closest to the provided
         timestamp.
         In case of a tie, the visit with largest id is selected.
@@ -820,13 +826,13 @@ class StorageInterface:
             visit_date: expected visit date
 
         Returns:
-            A visit
+            A visit if found, None otherwise
 
         """
         ...
 
     @remote_api_endpoint("origin/visit/getby")
-    def origin_visit_get_by(self, origin: str, visit: int) -> Optional[Dict[str, Any]]:
+    def origin_visit_get_by(self, origin: str, visit: int) -> Optional[OriginVisit]:
         """Retrieve origin visit's information.
 
         Args:
@@ -834,7 +840,7 @@ class StorageInterface:
             visit: visit id
 
         Returns:
-            The information on that particular (origin, visit) or None if
+            The information on that particular OriginVisit or None if
             it does not exist
 
         """
@@ -847,7 +853,7 @@ class StorageInterface:
         type: Optional[str] = None,
         allowed_statuses: Optional[List[str]] = None,
         require_snapshot: bool = False,
-    ) -> Optional[Dict[str, Any]]:
+    ) -> Optional[OriginVisit]:
         """Get the latest origin visit for the given origin, optionally
         looking only for those with one of the given allowed_statuses
         or for those with a snapshot.
@@ -864,16 +870,9 @@ class StorageInterface:
                 will be returned.
 
         Returns:
-            dict: a dict with the following keys:
-
-                - **origin**: the URL of the origin
-                - **visit**: origin visit id
-                - **type**: type of loader used for the visit
-                - **date**: timestamp of such visit
-                - **status**: Visit's new status
-                - **metadata**: Data associated to the visit
-                - **snapshot** (Optional[sha1_git]): identifier of the snapshot
-                    associated to the visit
+            OriginVisit matching the criteria if found, None otherwise. Note that as
+            OriginVisit no longer held reference on the visit status or snapshot, you
+            may want to use origin_visit_status_get_latest for those information.
 
         """
         ...
@@ -906,14 +905,16 @@ class StorageInterface:
         """
         ...
 
-    @remote_api_endpoint("origin/visit/get_random")
-    def origin_visit_get_random(self, type: str) -> Optional[Dict[str, Any]]:
+    @remote_api_endpoint("origin/visit_status/get_random")
+    def origin_visit_status_get_random(
+        self, type: str
+    ) -> Optional[Tuple[OriginVisit, OriginVisitStatus]]:
         """Randomly select one successful origin visit with <type>
         made in the last 3 months.
 
         Returns:
-            dict representing an origin visit, in the same format as
-            :py:meth:`origin_visit_get`.
+            One random tuple of (OriginVisit, OriginVisitStatus) matching the
+            selection criteria
 
         """
         ...
@@ -936,28 +937,15 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("origin/get")
-    def origin_get(self, origins):
-        """Return origins, either all identified by their ids or all
-        identified by tuples (type, url).
-
-        If the url is given and the type is omitted, one of the origins with
-        that url is returned.
+    def origin_get(self, origins: Iterable[str]) -> Iterable[Optional[Origin]]:
+        """Return origins.
 
         Args:
-            origin: a list of dictionaries representing the individual
-                origins to find.
-                These dicts have the key url:
-
-                - url (bytes): the url the origin points to
+            origin: a list of urls to find
 
         Returns:
-            dict: the origin dictionary with the keys:
-
-            - id: origin's id
-            - url: origin's url
-
-        Raises:
-            ValueError: if the url or the id don't exist.
+            the list of associated existing origin model objects. The unknown origins
+            will be returned as None at the same index as the input.
 
         """
         ...
@@ -1074,25 +1062,6 @@ class StorageInterface:
         """
         ...
 
-    @deprecated
-    @remote_api_endpoint("origin/add")
-    def origin_add_one(self, origin: Origin) -> str:
-        """Add origin to the storage
-
-        Args:
-            origin: dictionary representing the individual origin to add. This
-                dict has the following keys:
-
-                - type (FIXME: enum TBD): the origin type ('git', 'wget', ...)
-                - url (bytes): the url the origin points to
-
-        Returns:
-            the id of the added origin, or of the identical one that already
-            exists.
-
-        """
-        ...
-
     def stat_counters(self):
         """compute statistics about the number of tuples in various tables
 
@@ -1107,50 +1076,40 @@ class StorageInterface:
         """Recomputes the statistics for `stat_counters`."""
         ...
 
-    @remote_api_endpoint("content/metadata/add")
-    def content_metadata_add(
-        self,
-        id: str,
-        context: Dict[str, Union[str, bytes, int]],
-        discovery_date: datetime.datetime,
-        authority: Dict[str, Any],
-        fetcher: Dict[str, Any],
-        format: str,
-        metadata: bytes,
+    @remote_api_endpoint("raw_extrinsic_metadata/add")
+    def raw_extrinsic_metadata_add(
+        self, metadata: Iterable[RawExtrinsicMetadata],
     ) -> None:
-        """Add a content_metadata for the content at discovery_date,
-        obtained using the `fetcher` from the `authority`.
+        """Add extrinsic metadata on objects (contents, directories, ...).
 
         The authority and fetcher must be known to the storage before
         using this endpoint.
 
-        If there is already content metadata for the same content, authority,
+        If there is already metadata for the same object, authority,
         fetcher, and at the same date; the new one will be either dropped or
         will replace the existing one
         (it is unspecified which one of these two behaviors happens).
 
         Args:
-            discovery_date: when the metadata was fetched.
-            authority: a dict containing keys `type` and `url`.
-            fetcher: a dict containing keys `name` and `version`.
-            format: text field indicating the format of the content of the
-            metadata: blob of raw metadata
+            metadata: iterable of RawExtrinsicMetadata objects to be inserted.
         """
         ...
 
-    @remote_api_endpoint("content/metadata/get")
-    def content_metadata_get(
+    @remote_api_endpoint("raw_extrinsic_metadata/get")
+    def raw_extrinsic_metadata_get(
         self,
-        id: str,
-        authority: Dict[str, str],
+        object_type: MetadataTargetType,
+        id: Union[str, SWHID],
+        authority: MetadataAuthority,
         after: Optional[datetime.datetime] = None,
         page_token: Optional[bytes] = None,
         limit: int = 1000,
-    ) -> Dict[str, Any]:
-        """Retrieve list of all content_metadata entries for the id
+    ) -> Dict[str, Union[Optional[bytes], List[RawExtrinsicMetadata]]]:
+        """Retrieve list of all raw_extrinsic_metadata entries for the id
 
         Args:
-            id: the content's SWHID
+            object_type: one of the values of swh.model.model.MetadataTargetType
+            id: an URL if object_type is 'origin', else a core SWHID
             authority: a dict containing keys `type` and `url`.
             after: minimum discovery_date for a result to be returned
             page_token: opaque token, used to get the next page of results
@@ -1160,108 +1119,30 @@ class StorageInterface:
             dict with keys `next_page_token` and `results`.
             `next_page_token` is an opaque token that is used to get the
             next page of results, or `None` if there are no more results.
-            `results` is a list of dicts in the format:
-
-            .. code-block: python
-
-                {
-                    'authority': {'type': ..., 'url': ...},
-                    'fetcher': {'name': ..., 'version': ...},
-                    'discovery_date': ...,
-                    'format': '...',
-                    'metadata': b'...',
-                    'context': { ... },
-                }
+            `results` is a list of RawExtrinsicMetadata objects:
 
         """
         ...
 
-    @remote_api_endpoint("origin/metadata/add")
-    def origin_metadata_add(
-        self,
-        origin_url: str,
-        discovery_date: datetime.datetime,
-        authority: Dict[str, Any],
-        fetcher: Dict[str, Any],
-        format: str,
-        metadata: bytes,
-    ) -> None:
-        """Add an origin_metadata for the origin at discovery_date,
-        obtained using the `fetcher` from the `authority`.
+    @remote_api_endpoint("metadata_fetcher/add")
+    def metadata_fetcher_add(self, fetchers: Iterable[MetadataFetcher],) -> None:
+        """Add new metadata fetchers to the storage.
 
-        The authority and fetcher must be known to the storage before
-        using this endpoint.
-
-        If there is already origin metadata for the same origin, authority,
-        fetcher, and at the same date; the new one will be either dropped or
-        will replace the existing one
-        (it is unspecified which one of these two behaviors happens).
-
-        Args:
-            discovery_date: when the metadata was fetched.
-            authority: a dict containing keys `type` and `url`.
-            fetcher: a dict containing keys `name` and `version`.
-            format: text field indicating the format of the content of the
-            metadata: blob of raw metadata
-        """
-        ...
-
-    @remote_api_endpoint("origin/metadata/get")
-    def origin_metadata_get(
-        self,
-        origin_url: str,
-        authority: Dict[str, str],
-        after: Optional[datetime.datetime] = None,
-        page_token: Optional[bytes] = None,
-        limit: int = 1000,
-    ) -> Dict[str, Any]:
-        """Retrieve list of all origin_metadata entries for the origin_url
-
-        Args:
-            origin_url: the origin's URL
-            authority: a dict containing keys `type` and `url`.
-            after: minimum discovery_date for a result to be returned
-            page_token: opaque token, used to get the next page of results
-            limit: maximum number of results to be returned
-
-        Returns:
-            dict with keys `next_page_token` and `results`.
-            `next_page_token` is an opaque token that is used to get the
-            next page of results, or `None` if there are no more results.
-            `results` is a list of dicts in the format:
-
-            .. code-block: python
-
-                {
-                    'authority': {'type': ..., 'url': ...},
-                    'fetcher': {'name': ..., 'version': ...},
-                    'discovery_date': ...,
-                    'format': '...',
-                    'metadata': b'...'
-                }
-
-        """
-        ...
-
-    @remote_api_endpoint("fetcher/add")
-    def metadata_fetcher_add(
-        self, name: str, version: str, metadata: Dict[str, Any]
-    ) -> None:
-        """Add a new metadata fetcher to the storage.
-
-        `name` and `version` together are a unique identifier of this
+        Their `name` and `version` together are unique identifiers of this
         fetcher; and `metadata` is an arbitrary dict of JSONable data
-        with information about this fetcher.
+        with information about this fetcher, which must not be `None`
+        (but may be empty).
 
         Args:
-            name: the name of the fetcher
-            version: version of the fetcher
+            fetchers: iterable of MetadataFetcher to be inserted
 
         """
         ...
 
-    @remote_api_endpoint("fetcher/get")
-    def metadata_fetcher_get(self, name: str, version: str) -> Optional[Dict[str, Any]]:
+    @remote_api_endpoint("metadata_fetcher/get")
+    def metadata_fetcher_get(
+        self, name: str, version: str
+    ) -> Optional[MetadataFetcher]:
         """Retrieve information about a fetcher
 
         Args:
@@ -1269,36 +1150,39 @@ class StorageInterface:
             version: version of the fetcher
 
         Returns:
-            dictionary with keys `name`, `version`, and `metadata`; or None
-            if the fetcher is not known
+            a MetadataFetcher object (with a non-None metadata field) if it is known,
+            else None.
 
         """
         ...
 
-    @remote_api_endpoint("authority/add")
-    def metadata_authority_add(
-        self, type: str, url: str, metadata: Dict[str, Any]
-    ) -> None:
-        """Add a metadata authority
+    @remote_api_endpoint("metadata_authority/add")
+    def metadata_authority_add(self, authorities: Iterable[MetadataAuthority]) -> None:
+        """Add new metadata authorities to the storage.
+
+        Their `type` and `url` together are unique identifiers of this
+        authority; and `metadata` is an arbitrary dict of JSONable data
+        with information about this authority, which must not be `None`
+        (but may be empty).
 
         Args:
-            type: one of "deposit", "forge", or "registry"
-            url: unique URI identifying the authority
-            metadata: JSON-encodable object
+            authorities: iterable of MetadataAuthority to be inserted
         """
         ...
 
-    @remote_api_endpoint("authority/get")
-    def metadata_authority_get(self, type: str, url: str) -> Optional[Dict[str, Any]]:
+    @remote_api_endpoint("metadata_authority/get")
+    def metadata_authority_get(
+        self, type: MetadataAuthorityType, url: str
+    ) -> Optional[MetadataAuthority]:
         """Retrieve information about an authority
 
         Args:
-            type: one of "deposit", "forge", or "registry"
+            type: one of "deposit_client", "forge", or "registry"
             url: unique URI identifying the authority
 
         Returns:
-            dictionary with keys `type`, `url`, and `metadata`; or None
-            if the authority is not known
+            a MetadataAuthority object (with a non-None metadata field) if it is known,
+            else None.
         """
         ...
 
