@@ -9,8 +9,16 @@ import itertools
 
 from collections import defaultdict
 from contextlib import contextmanager
-from deprecated import deprecated
-from typing import Any, Dict, Iterable, List, Optional, Union
+from typing import (
+    Any,
+    Counter,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Tuple,
+    Union,
+)
 
 import attr
 import psycopg2
@@ -18,6 +26,7 @@ import psycopg2.pool
 import psycopg2.errors
 
 from swh.core.api.serializers import msgpack_loads, msgpack_dumps
+from swh.model.identifiers import parse_swhid, SWHID
 from swh.model.model import (
     Content,
     Directory,
@@ -29,23 +38,23 @@ from swh.model.model import (
     SkippedContent,
     Snapshot,
     SHA1_SIZE,
+    MetadataAuthority,
+    MetadataAuthorityType,
+    MetadataFetcher,
+    MetadataTargetType,
+    RawExtrinsicMetadata,
 )
 from swh.model.hashutil import DEFAULT_ALGORITHMS, hash_to_bytes, hash_to_hex
 from swh.storage.objstorage import ObjStorage
-from swh.storage.validate import VALIDATION_EXCEPTIONS
 from swh.storage.utils import now
 
 from . import converters
-from .extrinsic_metadata import (
-    check_extrinsic_metadata_context,
-    CONTEXT_KEYS,
-)
 from .common import db_transaction_generator, db_transaction
 from .db import Db
 from .exc import StorageArgumentException, StorageDBError, HashCollision
 from .algos import diff
 from .metrics import timed, send_metric, process_metrics
-from .utils import get_partition_bounds_bytes, extract_collision_hash
+from .utils import get_partition_bounds_bytes, extract_collision_hash, map_optional
 from .writer import JournalWriter
 
 
@@ -56,14 +65,17 @@ EMPTY_SNAPSHOT_ID = hash_to_bytes("1a8893e6a86f444e8be8e7bda6cb34fb1735a00e")
 """Identifier for the empty snapshot"""
 
 
-VALIDATION_EXCEPTIONS = VALIDATION_EXCEPTIONS + [
+VALIDATION_EXCEPTIONS = (
+    KeyError,
+    TypeError,
+    ValueError,
     psycopg2.errors.CheckViolation,
     psycopg2.errors.IntegrityError,
     psycopg2.errors.InvalidTextRepresentation,
     psycopg2.errors.NotNullViolation,
     psycopg2.errors.NumericValueOutOfRange,
     psycopg2.errors.UndefinedFunction,  # (raised on wrong argument typs)
-]
+)
 """Exceptions raised by postgresql when validation of the arguments
 failed."""
 
@@ -381,23 +393,6 @@ class Storage:
 
         return d
 
-    @staticmethod
-    def _skipped_content_validate(d):
-        """Sanity checks on status / reason / length, that postgresql
-        doesn't enforce."""
-        if d["status"] != "absent":
-            raise StorageArgumentException(
-                "Invalid content status: {}".format(d["status"])
-            )
-
-        if d.get("reason") is None:
-            raise StorageArgumentException(
-                "Must provide a reason if content is absent."
-            )
-
-        if d["length"] < -1:
-            raise StorageArgumentException("Content length must be positive or -1.")
-
     def _skipped_content_add_metadata(self, db, cur, content: Iterable[SkippedContent]):
         origin_ids = db.origin_id_get_by_url([cont.origin for cont in content], cur=cur)
         content = [
@@ -807,7 +802,7 @@ class Storage:
         self, visits: Iterable[OriginVisit], db=None, cur=None
     ) -> Iterable[OriginVisit]:
         for visit in visits:
-            origin = self.origin_get({"url": visit.origin}, db=db, cur=cur)
+            origin = self.origin_get([visit.origin], db=db, cur=cur)[0]
             if not origin:  # Cannot add a visit without an origin
                 raise StorageArgumentException("Unknown origin %s", visit.origin)
 
@@ -856,7 +851,7 @@ class Storage:
     ) -> None:
         # First round to check existence (fail early if any is ko)
         for visit_status in visit_statuses:
-            origin_url = self.origin_get({"url": visit_status.origin}, db=db, cur=cur)
+            origin_url = self.origin_get([visit_status.origin], db=db, cur=cur)[0]
             if not origin_url:
                 raise StorageArgumentException(f"Unknown origin {visit_status.origin}")
 
@@ -881,28 +876,6 @@ class Storage:
             return None
         return OriginVisitStatus.from_dict(row)
 
-    def _origin_visit_apply_update(
-        self, visit: Dict[str, Any], db, cur=None
-    ) -> Dict[str, Any]:
-        """Retrieve the latest visit status information for the origin visit.
-        Then merge it with the visit and return it.
-
-        """
-        visit_status = db.origin_visit_status_get_latest(
-            visit["origin"], visit["visit"], cur=cur
-        )
-        return {
-            # default to the values in visit
-            **visit,
-            # override with the last update
-            **visit_status,
-            # visit['origin'] is the URL (via a join), while
-            # visit_status['origin'] is only an id.
-            "origin": visit["origin"],
-            # but keep the date of the creation of the origin visit
-            "date": visit["date"],
-        }
-
     @timed
     @db_transaction_generator(statement_timeout=500)
     def origin_visit_get(
@@ -919,28 +892,37 @@ class Storage:
             origin, last_visit=last_visit, limit=limit, order=order, cur=cur
         )
         for line in lines:
-            visit = dict(zip(db.origin_visit_get_cols, line))
-            yield self._origin_visit_apply_update(visit, db)
+            yield dict(zip(db.origin_visit_get_cols, line))
 
     @timed
     @db_transaction(statement_timeout=500)
     def origin_visit_find_by_date(
         self, origin: str, visit_date: datetime.datetime, db=None, cur=None
-    ) -> Optional[Dict[str, Any]]:
-        visit = db.origin_visit_find_by_date(origin, visit_date, cur=cur)
-        if visit:
-            return self._origin_visit_apply_update(visit, db)
-        return None
+    ) -> Optional[OriginVisit]:
+        row_d = db.origin_visit_find_by_date(origin, visit_date, cur=cur)
+        if not row_d:
+            return None
+        return OriginVisit(
+            origin=row_d["origin"],
+            visit=row_d["visit"],
+            date=row_d["date"],
+            type=row_d["type"],
+        )
 
     @timed
     @db_transaction(statement_timeout=500)
     def origin_visit_get_by(
         self, origin: str, visit: int, db=None, cur=None
-    ) -> Optional[Dict[str, Any]]:
+    ) -> Optional[OriginVisit]:
         row = db.origin_visit_get(origin, visit, cur)
         if row:
-            visit_dict = dict(zip(db.origin_visit_get_cols, row))
-            return self._origin_visit_apply_update(visit_dict, db)
+            row_d = dict(zip(db.origin_visit_get_cols, row))
+            return OriginVisit(
+                origin=row_d["origin"],
+                visit=row_d["visit"],
+                date=row_d["date"],
+                type=row_d["type"],
+            )
         return None
 
     @timed
@@ -953,7 +935,7 @@ class Storage:
         require_snapshot: bool = False,
         db=None,
         cur=None,
-    ) -> Optional[Dict[str, Any]]:
+    ) -> Optional[OriginVisit]:
         row = db.origin_visit_get_latest(
             origin,
             type=type,
@@ -962,19 +944,39 @@ class Storage:
             cur=cur,
         )
         if row:
-            visit = dict(zip(db.origin_visit_get_cols, row))
-            return self._origin_visit_apply_update(visit, db)
+            row_d = dict(zip(db.origin_visit_get_cols, row))
+            visit = OriginVisit(
+                origin=row_d["origin"],
+                visit=row_d["visit"],
+                date=row_d["date"],
+                type=row_d["type"],
+            )
+            return visit
         return None
 
     @timed
     @db_transaction()
-    def origin_visit_get_random(
+    def origin_visit_status_get_random(
         self, type: str, db=None, cur=None
-    ) -> Optional[Dict[str, Any]]:
+    ) -> Optional[Tuple[OriginVisit, OriginVisitStatus]]:
         row = db.origin_visit_get_random(type, cur)
-        if row:
-            visit = dict(zip(db.origin_visit_get_cols, row))
-            return self._origin_visit_apply_update(visit, db)
+        if row is not None:
+            row_d = dict(zip(db.origin_visit_get_cols, row))
+            visit = OriginVisit(
+                origin=row_d["origin"],
+                visit=row_d["visit"],
+                date=row_d["date"],
+                type=row_d["type"],
+            )
+            visit_status = OriginVisitStatus(
+                origin=row_d["origin"],
+                visit=row_d["visit"],
+                date=row_d["date"],
+                status=row_d["status"],
+                metadata=row_d["metadata"],
+                snapshot=row_d["snapshot"],
+            )
+            return visit, visit_status
         return None
 
     @timed
@@ -992,28 +994,17 @@ class Storage:
 
     @timed
     @db_transaction(statement_timeout=500)
-    def origin_get(self, origins, db=None, cur=None):
-        if isinstance(origins, dict):
-            # Old API
-            return_single = True
-            origins = [origins]
-        elif len(origins) == 0:
-            return []
-        else:
-            return_single = False
-
-        origin_urls = [origin["url"] for origin in origins]
-        results = db.origin_get_by_url(origin_urls, cur)
-
-        results = [dict(zip(db.origin_cols, result)) for result in results]
-        if return_single:
-            assert len(results) == 1
-            if results[0]["url"] is not None:
-                return results[0]
-            else:
-                return None
-        else:
-            return [None if res["url"] is None else res for res in results]
+    def origin_get(
+        self, origins: Iterable[str], db=None, cur=None
+    ) -> Iterable[Optional[Origin]]:
+        origin_urls = list(origins)
+        rows = db.origin_get_by_url(origin_urls, cur)
+        result: List[Optional[Origin]] = []
+        for row in rows:
+            origin_d = dict(zip(db.origin_cols, row))
+            url = origin_d["url"]
+            result.append(None if url is None else Origin(url=url))
+        return result
 
     @timed
     @db_transaction_generator(statement_timeout=500)
@@ -1097,13 +1088,6 @@ class Storage:
                 added += 1
         return {"origin:add": added}
 
-    @deprecated("Use origin_add([origin]) instead")
-    @timed
-    @db_transaction()
-    def origin_add_one(self, origin: Origin, db=None, cur=None) -> str:
-        self.origin_add([origin])
-        return origin.url
-
     @db_transaction(statement_timeout=500)
     def stat_counters(self, db=None, cur=None):
         return {k: v for (k, v) in db.stat_counters()}
@@ -1129,150 +1113,66 @@ class Storage:
         for key in keys:
             cur.execute("select * from swh_update_counter(%s)", (key,))
 
-    @timed
     @db_transaction()
-    def content_metadata_add(
-        self,
-        id: str,
-        context: Dict[str, Union[str, bytes, int]],
-        discovery_date: datetime.datetime,
-        authority: Dict[str, Any],
-        fetcher: Dict[str, Any],
-        format: str,
-        metadata: bytes,
-        db=None,
-        cur=None,
+    def raw_extrinsic_metadata_add(
+        self, metadata: Iterable[RawExtrinsicMetadata], db, cur,
     ) -> None:
-        self._object_metadata_add(
-            "content",
-            id,
-            context,
-            discovery_date,
-            authority,
-            fetcher,
-            format,
-            metadata,
-            db,
-            cur,
-        )
+        counter = Counter[MetadataTargetType]()
+        for metadata_entry in metadata:
+            authority_id = self._get_authority_id(metadata_entry.authority, db, cur)
+            fetcher_id = self._get_fetcher_id(metadata_entry.fetcher, db, cur)
 
-    @timed
-    @db_transaction()
-    def content_metadata_get(
-        self,
-        id: str,
-        authority: Dict[str, str],
-        after: Optional[datetime.datetime] = None,
-        page_token: Optional[bytes] = None,
-        limit: int = 1000,
-        db=None,
-        cur=None,
-    ) -> Dict[str, Any]:
-        return self._object_metadata_get(
-            "content", id, authority, after, page_token, limit, db, cur
-        )
+            db.raw_extrinsic_metadata_add(
+                object_type=metadata_entry.type.value,
+                id=str(metadata_entry.id),
+                discovery_date=metadata_entry.discovery_date,
+                authority_id=authority_id,
+                fetcher_id=fetcher_id,
+                format=metadata_entry.format,
+                metadata=metadata_entry.metadata,
+                origin=metadata_entry.origin,
+                visit=metadata_entry.visit,
+                snapshot=map_optional(str, metadata_entry.snapshot),
+                release=map_optional(str, metadata_entry.release),
+                revision=map_optional(str, metadata_entry.revision),
+                path=metadata_entry.path,
+                directory=map_optional(str, metadata_entry.directory),
+                cur=cur,
+            )
+            counter[metadata_entry.type] += 1
 
-    @timed
-    @db_transaction()
-    def origin_metadata_add(
-        self,
-        origin_url: str,
-        discovery_date: datetime.datetime,
-        authority: Dict[str, Any],
-        fetcher: Dict[str, Any],
-        format: str,
-        metadata: bytes,
-        db=None,
-        cur=None,
-    ) -> None:
-        context: Dict[str, Union[str, bytes, int]] = {}  # origins have no context
-
-        self._object_metadata_add(
-            "origin",
-            origin_url,
-            context,
-            discovery_date,
-            authority,
-            fetcher,
-            format,
-            metadata,
-            db,
-            cur,
-        )
-
-    @timed
-    @db_transaction(statement_timeout=500)
-    def origin_metadata_get(
-        self,
-        origin_url: str,
-        authority: Dict[str, str],
-        after: Optional[datetime.datetime] = None,
-        page_token: Optional[bytes] = None,
-        limit: int = 1000,
-        db=None,
-        cur=None,
-    ) -> Dict[str, Any]:
-        result = self._object_metadata_get(
-            "origin", origin_url, authority, after, page_token, limit, db, cur
-        )
-
-        for res in result["results"]:
-            res.pop("id")
-            res["origin_url"] = origin_url
-
-        return result
-
-    def _object_metadata_add(
-        self,
-        object_type: str,
-        id: str,
-        context: Dict[str, Union[str, bytes, int]],
-        discovery_date: datetime.datetime,
-        authority: Dict[str, Any],
-        fetcher: Dict[str, Any],
-        format: str,
-        metadata: bytes,
-        db,
-        cur,
-    ) -> None:
-        check_extrinsic_metadata_context(object_type, context)
-
-        authority_id = self._get_authority_id(authority, db, cur)
-        fetcher_id = self._get_fetcher_id(fetcher, db, cur)
-        if not isinstance(metadata, bytes):
-            raise StorageArgumentException(
-                "metadata must be bytes, not %r" % (metadata,)
+        for (object_type, count) in counter.items():
+            send_metric(
+                f"{object_type.value}_metadata:add",
+                count=count,
+                method_name=f"{object_type.value}_metadata_add",
             )
 
-        db.object_metadata_add(
-            object_type,
-            id,
-            context,
-            discovery_date,
-            authority_id,
-            fetcher_id,
-            format,
-            metadata,
-            cur,
-        )
-
-        send_metric(
-            f"{object_type}_metadata:add",
-            count=1,
-            method_name=f"{object_type}_metadata_add",
-        )
-
-    def _object_metadata_get(
+    @db_transaction()
+    def raw_extrinsic_metadata_get(
         self,
-        object_type: str,
-        id: str,
-        authority: Dict[str, str],
-        after: Optional[datetime.datetime],
-        page_token: Optional[bytes],
-        limit: int,
-        db,
-        cur,
-    ) -> Dict[str, Any]:
+        object_type: MetadataTargetType,
+        id: Union[str, SWHID],
+        authority: MetadataAuthority,
+        after: Optional[datetime.datetime] = None,
+        page_token: Optional[bytes] = None,
+        limit: int = 1000,
+        db=None,
+        cur=None,
+    ) -> Dict[str, Union[Optional[bytes], List[RawExtrinsicMetadata]]]:
+        if object_type == MetadataTargetType.ORIGIN:
+            if isinstance(id, SWHID):
+                raise StorageArgumentException(
+                    f"raw_extrinsic_metadata_get called with object_type='origin', "
+                    f"but provided id is an SWHID: {id!r}"
+                )
+        else:
+            if not isinstance(id, SWHID):
+                raise StorageArgumentException(
+                    f"raw_extrinsic_metadata_get called with object_type!='origin', "
+                    f"but provided id is not an SWHID: {id!r}"
+                )
+
         if page_token:
             (after_time, after_fetcher) = msgpack_loads(page_token)
             if after and after_time < after:
@@ -1283,46 +1183,52 @@ class Storage:
             after_time = after
             after_fetcher = None
 
-        authority_id = db.metadata_authority_get_id(
-            authority["type"], authority["url"], cur
-        )
+        authority_id = self._get_authority_id(authority, db, cur)
         if not authority_id:
             return {
                 "next_page_token": None,
                 "results": [],
             }
 
-        rows = db.object_metadata_get(
-            object_type, id, authority_id, after_time, after_fetcher, limit + 1, cur
+        rows = db.raw_extrinsic_metadata_get(
+            object_type,
+            str(id),
+            authority_id,
+            after_time,
+            after_fetcher,
+            limit + 1,
+            cur,
         )
-        rows = [dict(zip(db.object_metadata_get_cols, row)) for row in rows]
+        rows = [dict(zip(db.raw_extrinsic_metadata_get_cols, row)) for row in rows]
         results = []
         for row in rows:
             row = row.copy()
             row.pop("metadata_fetcher.id")
-            context = {}
-            for key in CONTEXT_KEYS[object_type]:
-                value = row[key]
-                if value is not None:
-                    context[key] = value
 
-            result = {
-                "id": row["id"],
-                "authority": {
-                    "type": row.pop("metadata_authority.type"),
-                    "url": row.pop("metadata_authority.url"),
-                },
-                "fetcher": {
-                    "name": row.pop("metadata_fetcher.name"),
-                    "version": row.pop("metadata_fetcher.version"),
-                },
-                "discovery_date": row["discovery_date"],
-                "format": row["format"],
-                "metadata": row["metadata"],
-            }
+            assert str(id) == row["raw_extrinsic_metadata.id"]
 
-            if CONTEXT_KEYS[object_type]:
-                result["context"] = context
+            result = RawExtrinsicMetadata(
+                type=MetadataTargetType(row["raw_extrinsic_metadata.type"]),
+                id=id,
+                authority=MetadataAuthority(
+                    type=MetadataAuthorityType(row["metadata_authority.type"]),
+                    url=row["metadata_authority.url"],
+                ),
+                fetcher=MetadataFetcher(
+                    name=row["metadata_fetcher.name"],
+                    version=row["metadata_fetcher.version"],
+                ),
+                discovery_date=row["discovery_date"],
+                format=row["format"],
+                metadata=row["raw_extrinsic_metadata.metadata"],
+                origin=row["origin"],
+                visit=row["visit"],
+                snapshot=map_optional(parse_swhid, row["snapshot"]),
+                release=map_optional(parse_swhid, row["release"]),
+                revision=map_optional(parse_swhid, row["revision"]),
+                path=row["path"],
+                directory=map_optional(parse_swhid, row["directory"]),
+            )
 
             results.append(result)
 
@@ -1347,38 +1253,59 @@ class Storage:
     @timed
     @db_transaction()
     def metadata_fetcher_add(
-        self, name: str, version: str, metadata: Dict[str, Any], db=None, cur=None
+        self, fetchers: Iterable[MetadataFetcher], db=None, cur=None
     ) -> None:
-        db.metadata_fetcher_add(name, version, metadata)
-        send_metric("metadata_fetcher:add", count=1, method_name="metadata_fetcher")
+        count = 0
+        for fetcher in fetchers:
+            if fetcher.metadata is None:
+                raise StorageArgumentException(
+                    "MetadataFetcher.metadata may not be None in metadata_fetcher_add."
+                )
+            db.metadata_fetcher_add(
+                fetcher.name, fetcher.version, dict(fetcher.metadata), cur=cur
+            )
+            count += 1
+        send_metric("metadata_fetcher:add", count=count, method_name="metadata_fetcher")
 
     @timed
     @db_transaction(statement_timeout=500)
     def metadata_fetcher_get(
         self, name: str, version: str, db=None, cur=None
-    ) -> Optional[Dict[str, Any]]:
+    ) -> Optional[MetadataFetcher]:
         row = db.metadata_fetcher_get(name, version, cur=cur)
         if not row:
             return None
-        return dict(zip(db.metadata_fetcher_cols, row))
+        return MetadataFetcher.from_dict(dict(zip(db.metadata_fetcher_cols, row)))
 
     @timed
     @db_transaction()
     def metadata_authority_add(
-        self, type: str, url: str, metadata: Dict[str, Any], db=None, cur=None
+        self, authorities: Iterable[MetadataAuthority], db=None, cur=None
     ) -> None:
-        db.metadata_authority_add(type, url, metadata, cur)
-        send_metric("metadata_authority:add", count=1, method_name="metadata_authority")
+        count = 0
+        for authority in authorities:
+            if authority.metadata is None:
+                raise StorageArgumentException(
+                    "MetadataAuthority.metadata may not be None in "
+                    "metadata_authority_add."
+                )
+            db.metadata_authority_add(
+                authority.type.value, authority.url, dict(authority.metadata), cur=cur
+            )
+            count += 1
+        send_metric(
+            "metadata_authority:add", count=count, method_name="metadata_authority"
+        )
 
     @timed
     @db_transaction()
     def metadata_authority_get(
-        self, type: str, url: str, db=None, cur=None
-    ) -> Optional[Dict[str, Any]]:
-        row = db.metadata_authority_get(type, url, cur=cur)
+        self, type: MetadataAuthorityType, url: str, db=None, cur=None
+    ) -> Optional[MetadataAuthority]:
+        row = db.metadata_authority_get(type.value, url, cur=cur)
         if not row:
             return None
-        return dict(zip(db.metadata_authority_cols, row))
+        return MetadataAuthority.from_dict(dict(zip(db.metadata_authority_cols, row)))
 
     @timed
     def diff_directories(self, from_dir, to_dir, track_renaming=False):
@@ -1401,18 +1328,16 @@ class Storage:
     def flush(self, object_types: Optional[Iterable[str]] = None) -> Dict:
         return {}
 
-    def _get_authority_id(self, authority: Dict[str, Any], db, cur):
+    def _get_authority_id(self, authority: MetadataAuthority, db, cur):
         authority_id = db.metadata_authority_get_id(
-            authority["type"], authority["url"], cur
+            authority.type.value, authority.url, cur
         )
         if not authority_id:
             raise StorageArgumentException(f"Unknown authority {authority}")
         return authority_id
 
-    def _get_fetcher_id(self, fetcher: Dict[str, Any], db, cur):
-        fetcher_id = db.metadata_fetcher_get_id(
-            fetcher["name"], fetcher["version"], cur
-        )
+    def _get_fetcher_id(self, fetcher: MetadataFetcher, db, cur):
+        fetcher_id = db.metadata_fetcher_get_id(fetcher.name, fetcher.version, cur)
         if not fetcher_id:
             raise StorageArgumentException(f"Unknown fetcher {fetcher}")
         return fetcher_id
