@@ -20,8 +20,8 @@ def swh_storage():
     return get_storage(**storage_config)
 
 
-def test_filtering_proxy_storage_content(swh_storage, sample_data_model):
-    sample_content = sample_data_model["content"][0]
+def test_filtering_proxy_storage_content(swh_storage, sample_data):
+    sample_content = sample_data.content
 
     content = next(swh_storage.content_get([sample_content.sha1]))
     assert not content
@@ -42,8 +42,8 @@ def test_filtering_proxy_storage_content(swh_storage, sample_data_model):
     }
 
 
-def test_filtering_proxy_storage_skipped_content(swh_storage, sample_data_model):
-    sample_content = sample_data_model["skipped_content"][0]
+def test_filtering_proxy_storage_skipped_content(swh_storage, sample_data):
+    sample_content = sample_data.skipped_content
     sample_content_dict = sample_content.to_dict()
 
     content = next(swh_storage.skipped_content_missing([sample_content_dict]))
@@ -64,10 +64,10 @@ def test_filtering_proxy_storage_skipped_content(swh_storage, sample_data_model)
 
 
 def test_filtering_proxy_storage_skipped_content_missing_sha1_git(
-    swh_storage, sample_data_model
+    swh_storage, sample_data
 ):
     sample_contents = [
-        attr.evolve(c, sha1_git=None) for c in sample_data_model["skipped_content"]
+        attr.evolve(c, sha1_git=None) for c in sample_data.skipped_contents
     ]
     sample_content, sample_content2 = [c.to_dict() for c in sample_contents[:2]]
 
@@ -91,8 +91,8 @@ def test_filtering_proxy_storage_skipped_content_missing_sha1_git(
     assert content == []
 
 
-def test_filtering_proxy_storage_revision(swh_storage, sample_data_model):
-    sample_revision = sample_data_model["revision"][0]
+def test_filtering_proxy_storage_revision(swh_storage, sample_data):
+    sample_revision = sample_data.revision
 
     revision = next(swh_storage.revision_get([sample_revision.id]))
     assert not revision
@@ -111,8 +111,8 @@ def test_filtering_proxy_storage_revision(swh_storage, sample_data_model):
     }
 
 
-def test_filtering_proxy_storage_directory(swh_storage, sample_data_model):
-    sample_directory = sample_data_model["directory"][0]
+def test_filtering_proxy_storage_directory(swh_storage, sample_data):
+    sample_directory = sample_data.directory
 
     directory = next(swh_storage.directory_missing([sample_directory.id]))
     assert directory
