@@ -3,18 +3,17 @@
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
+import attr
+
 from unittest.mock import call
 
 import psycopg2
 import pytest
 
-from swh.model.model import OriginVisit
+from swh.model.model import MetadataTargetType
 
-from swh.storage import get_storage
 from swh.storage.exc import HashCollision, StorageArgumentException
 
-from .storage_data import date_visit1
-
 
 @pytest.fixture
 def monkeypatch_sleep(monkeypatch, swh_storage):
@@ -43,22 +42,11 @@ def swh_storage_backend_config():
     }
 
 
-@pytest.fixture
-def swh_storage(swh_storage_backend_config):
-    return get_storage(**swh_storage_backend_config)
-
-
-@pytest.fixture
-def swh_storage_validate(swh_storage_backend_config):
-    return get_storage(cls="validate", storage=swh_storage_backend_config)
-
-
-def test_retrying_proxy_storage_content_add(swh_storage, sample_data_model):
+def test_retrying_proxy_storage_content_add(swh_storage, sample_data):
     """Standard content_add works as before
 
     """
-    sample_content = sample_data_model["content"][0]
-
+    sample_content = sample_data.content
     content = next(swh_storage.content_get([sample_content.sha1]))
     assert not content
 
@@ -73,7 +61,7 @@ def test_retrying_proxy_storage_content_add(swh_storage, sample_data_model):
 
 
 def test_retrying_proxy_storage_content_add_with_retry(
-    monkeypatch_sleep, swh_storage, sample_data_model, mocker, fake_hash_collision,
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision,
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -88,7 +76,7 @@ def test_retrying_proxy_storage_content_add_with_retry(
         {"content:add": 1},
     ]
 
-    sample_content = sample_data_model["content"][0]
+    sample_content = sample_data.content
 
     content = next(swh_storage.content_get([sample_content.sha1]))
     assert not content
@@ -102,7 +90,7 @@ def test_retrying_proxy_storage_content_add_with_retry(
 
 
 def test_retrying_proxy_swh_storage_content_add_failure(
-    swh_storage, sample_data_model, mocker
+    swh_storage, sample_data, mocker
 ):
     """Unfiltered errors are raising without retry
 
@@ -110,7 +98,7 @@ def test_retrying_proxy_swh_storage_content_add_failure(
     mock_memory = mocker.patch("swh.storage.in_memory.InMemoryStorage.content_add")
     mock_memory.side_effect = StorageArgumentException("Refuse to add content always!")
 
-    sample_content = sample_data_model["content"][0]
+    sample_content = sample_data.content
 
     content = next(swh_storage.content_get([sample_content.sha1]))
     assert not content
@@ -121,17 +109,18 @@ def test_retrying_proxy_swh_storage_content_add_failure(
     assert mock_memory.call_count == 1
 
 
-def test_retrying_proxy_storage_content_add_metadata(swh_storage, sample_data_model):
+def test_retrying_proxy_storage_content_add_metadata(swh_storage, sample_data):
     """Standard content_add_metadata works as before
 
     """
-    sample_content = sample_data_model["content_metadata"][0]
+    sample_content = sample_data.content
+    content = attr.evolve(sample_content, data=None)
 
-    pk = sample_content.sha1
+    pk = content.sha1
     content_metadata = swh_storage.content_get_metadata([pk])
     assert not content_metadata[pk]
 
-    s = swh_storage.content_add_metadata([sample_content])
+    s = swh_storage.content_add_metadata([content])
     assert s == {
         "content:add": 1,
     }
@@ -142,7 +131,7 @@ def test_retrying_proxy_storage_content_add_metadata(swh_storage, sample_data_mo
 
 
 def test_retrying_proxy_storage_content_add_metadata_with_retry(
-    monkeypatch_sleep, swh_storage, sample_data_model, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -159,18 +148,19 @@ def test_retrying_proxy_storage_content_add_metadata_with_retry(
         {"content:add": 1},
     ]
 
-    sample_content = sample_data_model["content_metadata"][0]
+    sample_content = sample_data.content
+    content = attr.evolve(sample_content, data=None)
 
-    s = swh_storage.content_add_metadata([sample_content])
+    s = swh_storage.content_add_metadata([content])
     assert s == {"content:add": 1}
 
     mock_memory.assert_has_calls(
-        [call([sample_content]), call([sample_content]), call([sample_content]),]
+        [call([content]), call([content]), call([content]),]
     )
 
 
 def test_retrying_proxy_swh_storage_content_add_metadata_failure(
-    swh_storage, sample_data_model, mocker
+    swh_storage, sample_data, mocker
 ):
     """Unfiltered errors are raising without retry
 
@@ -182,23 +172,24 @@ def test_retrying_proxy_swh_storage_content_add_metadata_failure(
         "Refuse to add content_metadata!"
     )
 
-    sample_content = sample_data_model["content_metadata"][0]
-    pk = sample_content.sha1
+    sample_content = sample_data.content
+    content = attr.evolve(sample_content, data=None)
 
+    pk = content.sha1
     content_metadata = swh_storage.content_get_metadata([pk])
     assert not content_metadata[pk]
 
     with pytest.raises(StorageArgumentException, match="Refuse to add"):
-        swh_storage.content_add_metadata([sample_content])
+        swh_storage.content_add_metadata([content])
 
     assert mock_memory.call_count == 1
 
 
-def test_retrying_proxy_storage_skipped_content_add(swh_storage, sample_data_model):
+def test_retrying_proxy_storage_skipped_content_add(swh_storage, sample_data):
     """Standard skipped_content_add works as before
 
     """
-    sample_content = sample_data_model["skipped_content"][0]
+    sample_content = sample_data.skipped_content
     sample_content_dict = sample_content.to_dict()
 
     skipped_contents = list(swh_storage.skipped_content_missing([sample_content_dict]))
@@ -214,7 +205,7 @@ def test_retrying_proxy_storage_skipped_content_add(swh_storage, sample_data_mod
 
 
 def test_retrying_proxy_storage_skipped_content_add_with_retry(
-    monkeypatch_sleep, swh_storage, sample_data_model, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -230,7 +221,7 @@ def test_retrying_proxy_storage_skipped_content_add_with_retry(
         {"skipped_content:add": 1},
     ]
 
-    sample_content = sample_data_model["skipped_content"][0]
+    sample_content = sample_data.skipped_content
 
     s = swh_storage.skipped_content_add([sample_content])
     assert s == {"skipped_content:add": 1}
@@ -241,7 +232,7 @@ def test_retrying_proxy_storage_skipped_content_add_with_retry(
 
 
 def test_retrying_proxy_swh_storage_skipped_content_add_failure(
-    swh_storage, sample_data_model, mocker
+    swh_storage, sample_data, mocker
 ):
     """Unfiltered errors are raising without retry
 
@@ -253,7 +244,7 @@ def test_retrying_proxy_swh_storage_skipped_content_add_failure(
         "Refuse to add content_metadata!"
     )
 
-    sample_content = sample_data_model["skipped_content"][0]
+    sample_content = sample_data.skipped_content
     sample_content_dict = sample_content.to_dict()
 
     skipped_contents = list(swh_storage.skipped_content_missing([sample_content_dict]))
@@ -268,83 +259,19 @@ def test_retrying_proxy_swh_storage_skipped_content_add_failure(
     assert mock_memory.call_count == 1
 
 
-def test_retrying_proxy_swh_storage_origin_add_one(swh_storage, sample_data_model):
-    """Standard origin_add_one works as before
-
-    """
-    sample_origin = sample_data_model["origin"][0]
-    sample_origin_dict = sample_origin.to_dict()
-
-    origin = swh_storage.origin_get(sample_origin_dict)
-    assert not origin
-
-    swh_storage.origin_add_one(sample_origin)
-
-    origin = swh_storage.origin_get(sample_origin_dict)
-    assert origin["url"] == sample_origin.url
-
-
-def test_retrying_proxy_swh_storage_origin_add_one_retry(
-    monkeypatch_sleep, swh_storage, sample_data_model, mocker, fake_hash_collision
-):
-    """Multiple retries for hash collision and psycopg2 error but finally ok
-
-    """
-    sample_origin = sample_data_model["origin"][1]
-    mock_memory = mocker.patch("swh.storage.in_memory.InMemoryStorage.origin_add_one")
-    mock_memory.side_effect = [
-        # first try goes ko
-        fake_hash_collision,
-        # second try goes ko
-        psycopg2.IntegrityError("origin already inserted"),
-        # ok then!
-        sample_origin.url,
-    ]
-    sample_origin_dict = sample_origin.to_dict()
-
-    origin = swh_storage.origin_get(sample_origin_dict)
-    assert not origin
-
-    swh_storage.origin_add_one(sample_origin)
-
-    mock_memory.assert_has_calls(
-        [call(sample_origin), call(sample_origin), call(sample_origin),]
-    )
-
-
-def test_retrying_proxy_swh_storage_origin_add_one_failure(
-    swh_storage, sample_data_model, mocker
-):
-    """Unfiltered errors are raising without retry
-
-    """
-    mock_memory = mocker.patch("swh.storage.in_memory.InMemoryStorage.origin_add_one")
-    mock_memory.side_effect = StorageArgumentException("Refuse to add origin always!")
-
-    sample_origin = sample_data_model["origin"][0]
-    sample_origin_dict = sample_origin.to_dict()
-
-    origin = swh_storage.origin_get(sample_origin_dict)
-    assert not origin
-
-    with pytest.raises(StorageArgumentException, match="Refuse to add"):
-        swh_storage.origin_add_one(sample_origin)
-
-    assert mock_memory.call_count == 1
-
-
-def test_retrying_proxy_swh_storage_origin_visit_add(swh_storage, sample_data_model):
+def test_retrying_proxy_swh_storage_origin_visit_add(swh_storage, sample_data):
     """Standard origin_visit_add works as before
 
     """
-    origin = sample_data_model["origin"][0]
+    origin = sample_data.origin
+    visit = sample_data.origin_visit
+    assert visit.origin == origin.url
 
-    swh_storage.origin_add_one(origin)
+    swh_storage.origin_add([origin])
 
     origins = list(swh_storage.origin_visit_get(origin.url))
     assert not origins
 
-    visit = OriginVisit(origin=origin.url, date=date_visit1, type="hg")
     origin_visit = swh_storage.origin_visit_add([visit])[0]
     assert origin_visit.origin == origin.url
     assert isinstance(origin_visit.visit, int)
@@ -355,16 +282,18 @@ def test_retrying_proxy_swh_storage_origin_visit_add(swh_storage, sample_data_mo
 
 
 def test_retrying_proxy_swh_storage_origin_visit_add_retry(
-    monkeypatch_sleep, swh_storage, sample_data_model, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
     """
-    origin = sample_data_model["origin"][1]
-    swh_storage.origin_add_one(origin)
+    origin = sample_data.origin
+    visit = sample_data.origin_visit
+    assert visit.origin == origin.url
+
+    swh_storage.origin_add([origin])
 
     mock_memory = mocker.patch("swh.storage.in_memory.InMemoryStorage.origin_visit_add")
-    visit = OriginVisit(origin=origin.url, date=date_visit1, type="git")
     mock_memory.side_effect = [
         # first try goes ko
         fake_hash_collision,
@@ -386,7 +315,7 @@ def test_retrying_proxy_swh_storage_origin_visit_add_retry(
 
 
 def test_retrying_proxy_swh_storage_origin_visit_add_failure(
-    swh_storage, sample_data_model, mocker
+    swh_storage, sample_data, mocker
 ):
     """Unfiltered errors are raising without retry
 
@@ -394,13 +323,14 @@ def test_retrying_proxy_swh_storage_origin_visit_add_failure(
     mock_memory = mocker.patch("swh.storage.in_memory.InMemoryStorage.origin_visit_add")
     mock_memory.side_effect = StorageArgumentException("Refuse to add origin always!")
 
-    origin = sample_data_model["origin"][0]
+    origin = sample_data.origin
+    visit = sample_data.origin_visit
+    assert visit.origin == origin.url
 
     origins = list(swh_storage.origin_visit_get(origin.url))
     assert not origins
 
     with pytest.raises(StorageArgumentException, match="Refuse to add"):
-        visit = OriginVisit(origin=origin.url, date=date_visit1, type="svn",)
         swh_storage.origin_visit_add([visit])
 
     mock_memory.assert_has_calls(
@@ -408,32 +338,28 @@ def test_retrying_proxy_swh_storage_origin_visit_add_failure(
     )
 
 
-def test_retrying_proxy_storage_metadata_fetcher_add(swh_storage_validate, sample_data):
+def test_retrying_proxy_storage_metadata_fetcher_add(swh_storage, sample_data):
     """Standard metadata_fetcher_add works as before
 
     """
-    fetcher = sample_data["fetcher"][0]
+    fetcher = sample_data.metadata_fetcher
 
-    metadata_fetcher = swh_storage_validate.metadata_fetcher_get(
-        fetcher["name"], fetcher["version"]
-    )
+    metadata_fetcher = swh_storage.metadata_fetcher_get(fetcher.name, fetcher.version)
     assert not metadata_fetcher
 
-    swh_storage_validate.metadata_fetcher_add(**fetcher)
+    swh_storage.metadata_fetcher_add([fetcher])
 
-    actual_fetcher = swh_storage_validate.metadata_fetcher_get(
-        fetcher["name"], fetcher["version"]
-    )
+    actual_fetcher = swh_storage.metadata_fetcher_get(fetcher.name, fetcher.version)
     assert actual_fetcher == fetcher
 
 
 def test_retrying_proxy_storage_metadata_fetcher_add_with_retry(
-    monkeypatch_sleep, swh_storage_validate, sample_data, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision,
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
     """
-    fetcher = sample_data["fetcher"][0]
+    fetcher = sample_data.metadata_fetcher
     mock_memory = mocker.patch(
         "swh.storage.in_memory.InMemoryStorage.metadata_fetcher_add"
     )
@@ -446,24 +372,18 @@ def test_retrying_proxy_storage_metadata_fetcher_add_with_retry(
         [fetcher],
     ]
 
-    actual_fetcher = swh_storage_validate.metadata_fetcher_get(
-        fetcher["name"], fetcher["version"]
-    )
+    actual_fetcher = swh_storage.metadata_fetcher_get(fetcher.name, fetcher.version)
     assert not actual_fetcher
 
-    swh_storage_validate.metadata_fetcher_add(**fetcher)
+    swh_storage.metadata_fetcher_add([fetcher])
 
     mock_memory.assert_has_calls(
-        [
-            call(fetcher["name"], fetcher["version"], fetcher["metadata"]),
-            call(fetcher["name"], fetcher["version"], fetcher["metadata"]),
-            call(fetcher["name"], fetcher["version"], fetcher["metadata"]),
-        ]
+        [call([fetcher]), call([fetcher]), call([fetcher]),]
     )
 
 
 def test_retrying_proxy_swh_storage_metadata_fetcher_add_failure(
-    swh_storage_validate, sample_data, mocker
+    swh_storage, sample_data, mocker
 ):
     """Unfiltered errors are raising without retry
 
@@ -475,46 +395,38 @@ def test_retrying_proxy_swh_storage_metadata_fetcher_add_failure(
         "Refuse to add metadata_fetcher always!"
     )
 
-    fetcher = sample_data["fetcher"][0]
+    fetcher = sample_data.metadata_fetcher
 
-    actual_fetcher = swh_storage_validate.metadata_fetcher_get(
-        fetcher["name"], fetcher["version"]
-    )
+    actual_fetcher = swh_storage.metadata_fetcher_get(fetcher.name, fetcher.version)
     assert not actual_fetcher
 
     with pytest.raises(StorageArgumentException, match="Refuse to add"):
-        swh_storage_validate.metadata_fetcher_add(**fetcher)
+        swh_storage.metadata_fetcher_add([fetcher])
 
     assert mock_memory.call_count == 1
 
 
-def test_retrying_proxy_storage_metadata_authority_add(
-    swh_storage_validate, sample_data
-):
+def test_retrying_proxy_storage_metadata_authority_add(swh_storage, sample_data):
     """Standard metadata_authority_add works as before
 
     """
-    authority = sample_data["authority"][0]
+    authority = sample_data.metadata_authority
 
-    assert not swh_storage_validate.metadata_authority_get(
-        authority["type"], authority["url"]
-    )
+    assert not swh_storage.metadata_authority_get(authority.type, authority.url)
 
-    swh_storage_validate.metadata_authority_add(**authority)
+    swh_storage.metadata_authority_add([authority])
 
-    actual_authority = swh_storage_validate.metadata_authority_get(
-        authority["type"], authority["url"]
-    )
+    actual_authority = swh_storage.metadata_authority_get(authority.type, authority.url)
     assert actual_authority == authority
 
 
 def test_retrying_proxy_storage_metadata_authority_add_with_retry(
-    monkeypatch_sleep, swh_storage_validate, sample_data, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision,
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
     """
-    authority = sample_data["authority"][0]
+    authority = sample_data.metadata_authority
 
     mock_memory = mocker.patch(
         "swh.storage.in_memory.InMemoryStorage.metadata_authority_add"
@@ -528,21 +440,17 @@ def test_retrying_proxy_storage_metadata_authority_add_with_retry(
         None,
     ]
 
-    assert not swh_storage_validate.metadata_authority_get(
-        authority["type"], authority["url"]
-    )
+    assert not swh_storage.metadata_authority_get(authority.type, authority.url)
 
-    swh_storage_validate.metadata_authority_add(**authority)
+    swh_storage.metadata_authority_add([authority])
 
-    authority_arg_names = ("type", "url", "metadata")
-    authority_args = [authority[key] for key in authority_arg_names]
     mock_memory.assert_has_calls(
-        [call(*authority_args), call(*authority_args), call(*authority_args),]
+        [call([authority]), call([authority]), call([authority])]
     )
 
 
 def test_retrying_proxy_swh_storage_metadata_authority_add_failure(
-    swh_storage_validate, sample_data, mocker
+    swh_storage, sample_data, mocker
 ):
     """Unfiltered errors are raising without retry
 
@@ -554,51 +462,55 @@ def test_retrying_proxy_swh_storage_metadata_authority_add_failure(
         "Refuse to add authority_id always!"
     )
 
-    authority = sample_data["authority"][0]
+    authority = sample_data.metadata_authority
 
-    swh_storage_validate.metadata_authority_get(authority["type"], authority["url"])
+    swh_storage.metadata_authority_get(authority.type, authority.url)
 
     with pytest.raises(StorageArgumentException, match="Refuse to add"):
-        swh_storage_validate.metadata_authority_add(**authority)
+        swh_storage.metadata_authority_add([authority])
 
     assert mock_memory.call_count == 1
 
 
-def test_retrying_proxy_storage_origin_metadata_add(swh_storage_validate, sample_data):
-    """Standard origin_metadata_add works as before
+def test_retrying_proxy_storage_raw_extrinsic_metadata_add(swh_storage, sample_data):
+    """Standard raw_extrinsic_metadata_add works as before
 
     """
-    ori_meta = sample_data["origin_metadata"][0]
-    swh_storage_validate.origin_add_one({"url": ori_meta["origin_url"]})
-    swh_storage_validate.metadata_authority_add(**sample_data["authority"][0])
-    swh_storage_validate.metadata_fetcher_add(**sample_data["fetcher"][0])
+    origin = sample_data.origin
+    ori_meta = sample_data.origin_metadata1
+    assert origin.url == ori_meta.id
+    swh_storage.origin_add([origin])
+    swh_storage.metadata_authority_add([sample_data.metadata_authority])
+    swh_storage.metadata_fetcher_add([sample_data.metadata_fetcher])
 
-    origin_metadata = swh_storage_validate.origin_metadata_get(
-        ori_meta["origin_url"], ori_meta["authority"]
+    origin_metadata = swh_storage.raw_extrinsic_metadata_get(
+        MetadataTargetType.ORIGIN, ori_meta.id, ori_meta.authority
     )
     assert origin_metadata["next_page_token"] is None
     assert not origin_metadata["results"]
 
-    swh_storage_validate.origin_metadata_add(**ori_meta)
+    swh_storage.raw_extrinsic_metadata_add([ori_meta])
 
-    origin_metadata = swh_storage_validate.origin_metadata_get(
-        ori_meta["origin_url"], ori_meta["authority"]
+    origin_metadata = swh_storage.raw_extrinsic_metadata_get(
+        MetadataTargetType.ORIGIN, ori_meta.id, ori_meta.authority
     )
     assert origin_metadata
 
 
-def test_retrying_proxy_storage_origin_metadata_add_with_retry(
-    monkeypatch_sleep, swh_storage_validate, sample_data, mocker, fake_hash_collision
+def test_retrying_proxy_storage_raw_extrinsic_metadata_add_with_retry(
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision,
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
     """
-    ori_meta = sample_data["origin_metadata"][0]
-    swh_storage_validate.origin_add_one({"url": ori_meta["origin_url"]})
-    swh_storage_validate.metadata_authority_add(**sample_data["authority"][0])
-    swh_storage_validate.metadata_fetcher_add(**sample_data["fetcher"][0])
+    origin = sample_data.origin
+    ori_meta = sample_data.origin_metadata1
+    assert origin.url == ori_meta.id
+    swh_storage.origin_add([origin])
+    swh_storage.metadata_authority_add([sample_data.metadata_authority])
+    swh_storage.metadata_fetcher_add([sample_data.metadata_fetcher])
     mock_memory = mocker.patch(
-        "swh.storage.in_memory.InMemoryStorage.origin_metadata_add"
+        "swh.storage.in_memory.InMemoryStorage.raw_extrinsic_metadata_add"
     )
 
     mock_memory.side_effect = [
@@ -611,63 +523,44 @@ def test_retrying_proxy_storage_origin_metadata_add_with_retry(
     ]
 
     # No exception raised as insertion finally came through
-    swh_storage_validate.origin_metadata_add(**ori_meta)
+    swh_storage.raw_extrinsic_metadata_add([ori_meta])
 
     mock_memory.assert_has_calls(
         [  # 3 calls, as long as error raised
-            call(
-                ori_meta["origin_url"],
-                ori_meta["discovery_date"],
-                ori_meta["authority"],
-                ori_meta["fetcher"],
-                ori_meta["format"],
-                ori_meta["metadata"],
-            ),
-            call(
-                ori_meta["origin_url"],
-                ori_meta["discovery_date"],
-                ori_meta["authority"],
-                ori_meta["fetcher"],
-                ori_meta["format"],
-                ori_meta["metadata"],
-            ),
-            call(
-                ori_meta["origin_url"],
-                ori_meta["discovery_date"],
-                ori_meta["authority"],
-                ori_meta["fetcher"],
-                ori_meta["format"],
-                ori_meta["metadata"],
-            ),
+            call([ori_meta]),
+            call([ori_meta]),
+            call([ori_meta]),
         ]
     )
 
 
-def test_retrying_proxy_swh_storage_origin_metadata_add_failure(
-    swh_storage_validate, sample_data, mocker
+def test_retrying_proxy_swh_storage_raw_extrinsic_metadata_add_failure(
+    swh_storage, sample_data, mocker
 ):
     """Unfiltered errors are raising without retry
 
     """
     mock_memory = mocker.patch(
-        "swh.storage.in_memory.InMemoryStorage.origin_metadata_add"
+        "swh.storage.in_memory.InMemoryStorage.raw_extrinsic_metadata_add"
     )
     mock_memory.side_effect = StorageArgumentException("Refuse to add always!")
 
-    ori_meta = sample_data["origin_metadata"][0]
-    swh_storage_validate.origin_add_one({"url": ori_meta["origin_url"]})
+    origin = sample_data.origin
+    ori_meta = sample_data.origin_metadata1
+    assert origin.url == ori_meta.id
+    swh_storage.origin_add([origin])
 
     with pytest.raises(StorageArgumentException, match="Refuse to add"):
-        swh_storage_validate.origin_metadata_add(**ori_meta)
+        swh_storage.raw_extrinsic_metadata_add([ori_meta])
 
     assert mock_memory.call_count == 1
 
 
-def test_retrying_proxy_storage_directory_add(swh_storage, sample_data_model):
+def test_retrying_proxy_storage_directory_add(swh_storage, sample_data):
     """Standard directory_add works as before
 
     """
-    sample_dir = sample_data_model["directory"][0]
+    sample_dir = sample_data.directory
 
     directory = swh_storage.directory_get_random()  # no directory
     assert not directory
@@ -682,7 +575,7 @@ def test_retrying_proxy_storage_directory_add(swh_storage, sample_data_model):
 
 
 def test_retrying_proxy_storage_directory_add_with_retry(
-    monkeypatch_sleep, swh_storage, sample_data_model, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -697,7 +590,7 @@ def test_retrying_proxy_storage_directory_add_with_retry(
         {"directory:add": 1},
     ]
 
-    sample_dir = sample_data_model["directory"][1]
+    sample_dir = sample_data.directories[1]
 
     directory_id = swh_storage.directory_get_random()  # no directory
     assert not directory_id
@@ -713,7 +606,7 @@ def test_retrying_proxy_storage_directory_add_with_retry(
 
 
 def test_retrying_proxy_swh_storage_directory_add_failure(
-    swh_storage, sample_data_model, mocker
+    swh_storage, sample_data, mocker
 ):
     """Unfiltered errors are raising without retry
 
@@ -723,7 +616,7 @@ def test_retrying_proxy_swh_storage_directory_add_failure(
         "Refuse to add directory always!"
     )
 
-    sample_dir = sample_data_model["directory"][0]
+    sample_dir = sample_data.directory
 
     directory_id = swh_storage.directory_get_random()  # no directory
     assert not directory_id
@@ -734,11 +627,11 @@ def test_retrying_proxy_swh_storage_directory_add_failure(
     assert mock_memory.call_count == 1
 
 
-def test_retrying_proxy_storage_revision_add(swh_storage, sample_data_model):
+def test_retrying_proxy_storage_revision_add(swh_storage, sample_data):
     """Standard revision_add works as before
 
     """
-    sample_rev = sample_data_model["revision"][0]
+    sample_rev = sample_data.revision
 
     revision = next(swh_storage.revision_get([sample_rev.id]))
     assert not revision
@@ -753,7 +646,7 @@ def test_retrying_proxy_storage_revision_add(swh_storage, sample_data_model):
 
 
 def test_retrying_proxy_storage_revision_add_with_retry(
-    monkeypatch_sleep, swh_storage, sample_data_model, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -768,7 +661,7 @@ def test_retrying_proxy_storage_revision_add_with_retry(
         {"revision:add": 1},
     ]
 
-    sample_rev = sample_data_model["revision"][0]
+    sample_rev = sample_data.revision
 
     revision = next(swh_storage.revision_get([sample_rev.id]))
     assert not revision
@@ -784,7 +677,7 @@ def test_retrying_proxy_storage_revision_add_with_retry(
 
 
 def test_retrying_proxy_swh_storage_revision_add_failure(
-    swh_storage, sample_data_model, mocker
+    swh_storage, sample_data, mocker
 ):
     """Unfiltered errors are raising without retry
 
@@ -792,7 +685,7 @@ def test_retrying_proxy_swh_storage_revision_add_failure(
     mock_memory = mocker.patch("swh.storage.in_memory.InMemoryStorage.revision_add")
     mock_memory.side_effect = StorageArgumentException("Refuse to add revision always!")
 
-    sample_rev = sample_data_model["revision"][0]
+    sample_rev = sample_data.revision
 
     revision = next(swh_storage.revision_get([sample_rev.id]))
     assert not revision
@@ -803,11 +696,11 @@ def test_retrying_proxy_swh_storage_revision_add_failure(
     assert mock_memory.call_count == 1
 
 
-def test_retrying_proxy_storage_release_add(swh_storage, sample_data_model):
+def test_retrying_proxy_storage_release_add(swh_storage, sample_data):
     """Standard release_add works as before
 
     """
-    sample_rel = sample_data_model["release"][0]
+    sample_rel = sample_data.release
 
     release = next(swh_storage.release_get([sample_rel.id]))
     assert not release
@@ -822,7 +715,7 @@ def test_retrying_proxy_storage_release_add(swh_storage, sample_data_model):
 
 
 def test_retrying_proxy_storage_release_add_with_retry(
-    monkeypatch_sleep, swh_storage, sample_data_model, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -837,7 +730,7 @@ def test_retrying_proxy_storage_release_add_with_retry(
         {"release:add": 1},
     ]
 
-    sample_rel = sample_data_model["release"][0]
+    sample_rel = sample_data.release
 
     release = next(swh_storage.release_get([sample_rel.id]))
     assert not release
@@ -853,7 +746,7 @@ def test_retrying_proxy_storage_release_add_with_retry(
 
 
 def test_retrying_proxy_swh_storage_release_add_failure(
-    swh_storage, sample_data_model, mocker
+    swh_storage, sample_data, mocker
 ):
     """Unfiltered errors are raising without retry
 
@@ -861,7 +754,7 @@ def test_retrying_proxy_swh_storage_release_add_failure(
     mock_memory = mocker.patch("swh.storage.in_memory.InMemoryStorage.release_add")
     mock_memory.side_effect = StorageArgumentException("Refuse to add release always!")
 
-    sample_rel = sample_data_model["release"][0]
+    sample_rel = sample_data.release
 
     release = next(swh_storage.release_get([sample_rel.id]))
     assert not release
@@ -872,11 +765,11 @@ def test_retrying_proxy_swh_storage_release_add_failure(
     assert mock_memory.call_count == 1
 
 
-def test_retrying_proxy_storage_snapshot_add(swh_storage, sample_data_model):
+def test_retrying_proxy_storage_snapshot_add(swh_storage, sample_data):
     """Standard snapshot_add works as before
 
     """
-    sample_snap = sample_data_model["snapshot"][0]
+    sample_snap = sample_data.snapshot
 
     snapshot = swh_storage.snapshot_get(sample_snap.id)
     assert not snapshot
@@ -891,7 +784,7 @@ def test_retrying_proxy_storage_snapshot_add(swh_storage, sample_data_model):
 
 
 def test_retrying_proxy_storage_snapshot_add_with_retry(
-    monkeypatch_sleep, swh_storage, sample_data_model, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -906,7 +799,7 @@ def test_retrying_proxy_storage_snapshot_add_with_retry(
         {"snapshot:add": 1},
     ]
 
-    sample_snap = sample_data_model["snapshot"][0]
+    sample_snap = sample_data.snapshot
 
     snapshot = swh_storage.snapshot_get(sample_snap.id)
     assert not snapshot
@@ -922,7 +815,7 @@ def test_retrying_proxy_storage_snapshot_add_with_retry(
 
 
 def test_retrying_proxy_swh_storage_snapshot_add_failure(
-    swh_storage, sample_data_model, mocker
+    swh_storage, sample_data, mocker
 ):
     """Unfiltered errors are raising without retry
 
@@ -930,7 +823,7 @@ def test_retrying_proxy_swh_storage_snapshot_add_failure(
     mock_memory = mocker.patch("swh.storage.in_memory.InMemoryStorage.snapshot_add")
     mock_memory.side_effect = StorageArgumentException("Refuse to add snapshot always!")
 
-    sample_snap = sample_data_model["snapshot"][0]
+    sample_snap = sample_data.snapshot
 
     snapshot = swh_storage.snapshot_get(sample_snap.id)
     assert not snapshot
